# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import ast
import re
from unittest import mock

from neutronclient.common import exceptions as neutron_ex
import novaclient.exceptions as nova_ex
import six

from sahara.conductor import resource as r
from sahara.plugins.fake import plugin
import sahara.service.validation as v
from sahara.tests.unit import base
from sahara.tests.unit import testutils as tu
from sahara.utils import cluster as c_u

m = {}

_types_checks = {
    "string": [1, (), {}, True],
    "integer": ["a", (), {}, True],
    "uuid": ["z550e8400-e29b-41d4-a716-446655440000", 1, "a", (), {}, True],
    "array": [{}, 'a', 1, True],
    "boolean": [1, 'a', (), {}]
}


def _update_data(data, update):
    data.update(update)
    return data


def _get_plugins():
    fake = plugin.FakePluginProvider
    fake.name = 'fake'
    return [fake]


def _get_plugin(name):
    if name == 'fake':
        fake = plugin.FakePluginProvider
        fake.name = 'fake'
        return fake
    return None


def _get_keypair(name):
    if name != "test_keypair":
        raise nova_ex.NotFound("")


def _get_network(resource, id):
    if id != "d9a3bebc-f788-4b81-9a93-aa048022c1ca":
        raise neutron_ex.NotFound("")
    return 'OK'


def _get_fl_ip_pool_list():
    return [FakeNetwork("d9a3bebc-f788-4b81-9a93-aa048022c1ca")]


def _get_availability_zone_list(detailed=True):
    return [FakeAvailabilityZone('nova')]


def _get_heat_stack_list(**kwargs):
    if (kwargs.get('filters') and
            kwargs.get('filters').get('name') == 'test-heat'):
        return [FakeStack('test-heat')]
    return []


class FakeStack(object):
    def __init__(self, name):
        self.stack_name = name


class FakeNetwork(object):
    def __init__(self, name):
        self.name = name


class FakeAvailabilityZone(object):
    def __init__(self, name):
        self.zoneName = name


class FakeFlavor(object):
    def __init__(self, id):
        self.id = id


def _get_flavors_list():
    return [FakeFlavor("42")]


def _get_security_groups_list():
    return {'security_groups': [
            {"id": "1", "name": "default"},
            {"id": "2", "name": "group1"},
            {"id": "3", "name": "group2"}]}


def start_patch(patch_templates=True):
    get_clusters_p = mock.patch("sahara.service.api.v10.get_clusters")
    get_cluster_p = mock.patch("sahara.service.api.v10.get_cluster")
    if patch_templates:
        get_ng_templates_p = mock.patch(
            "sahara.service.api.v10.get_node_group_templates")
        get_ng_template_p = mock.patch(
            "sahara.service.api.v10.get_node_group_template")
    if patch_templates:
        get_cl_templates_p = mock.patch(
            "sahara.service.api.v10.get_cluster_templates")
        get_cl_template_p = mock.patch(
            "sahara.service.api.v10.get_cluster_template")
    nova_p = mock.patch("sahara.utils.openstack.nova.client")
    heat_p = mock.patch("sahara.utils.openstack.heat.client")
    image_manager_p = mock.patch(
        "sahara.utils.openstack.images.SaharaImageManager")
    cinder_p = mock.patch("sahara.utils.openstack.cinder.client")
    cinder_exists_p = mock.patch(
        "sahara.utils.openstack.cinder.check_cinder_exists")
    get_image_p = mock.patch("sahara.service.api.v10.get_image")

    get_image = get_image_p.start()
    get_clusters = get_clusters_p.start()
    get_cluster = get_cluster_p.start()
    if patch_templates:
        get_ng_templates = get_ng_templates_p.start()
        get_ng_template = get_ng_template_p.start()
    if patch_templates:
        get_cl_templates = get_cl_templates_p.start()
        get_cl_template_p.start()

    nova = nova_p.start()

    if patch_templates:
        get_cl_templates.return_value = []

    nova().flavors.list.side_effect = _get_flavors_list
    nova().keypairs.get.side_effect = _get_keypair
    nova().floating_ip_pools.list.side_effect = _get_fl_ip_pool_list
    nova().availability_zones.list.side_effect = _get_availability_zone_list

    neutron_p = mock.patch("sahara.utils.openstack.neutron.client")
    neutron = neutron_p.start()
    neutron().find_resource_by_id.side_effect = _get_network
    neutron().find_resource_by_id.__name__ = 'find_resource_by_id'
    neutron().list_security_groups.side_effect = _get_security_groups_list

    heat = heat_p.start()
    heat().stacks.list.side_effect = _get_heat_stack_list

    image_manager = image_manager_p.start()

    cinder = cinder_p.start()
    cinder().availability_zones.list.side_effect = _get_availability_zone_list

    cinder_exists = cinder_exists_p.start()
    cinder_exists.return_value = True

    class Image(object):
        def __init__(self, name='test'):
            self.name = name

        @property
        def id(self):
            if self.name == 'test':
                return '550e8400-e29b-41d4-a716-446655440000'
            else:
                return '813fe450-40d2-4acc-ade5-ea753a1bd5bc'

        @property
        def tags(self):
            if self.name == 'test':
                return ['fake', '0.1']
            else:
                return ['fake', 'wrong_tag']

    def _get_image(id):
        if id == '550e8400-e29b-41d4-a716-446655440000':
            return Image()
        else:
            return Image('wrong_test')

    get_image.side_effect = _get_image
    image_manager().list_registered.return_value = [Image(),
                                                    Image(name='wrong_name')]
    ng_dict = tu.make_ng_dict('ng', '42', ['namenode'], 1)
    cluster = tu.create_cluster('test', 't', 'fake', '0.1', [ng_dict],
                                id=1, status=c_u.CLUSTER_STATUS_ACTIVE)
    # stub clusters list
    get_clusters.return_value = [cluster]
    get_cluster.return_value = cluster

    # stub node templates
    if patch_templates:
        ngt_dict = {'name': 'test', 'tenant_id': 't', 'flavor_id': '42',
                    'plugin_name': 'fake', 'hadoop_version': '0.1',
                    'id': '550e8400-e29b-41d4-a716-446655440000',
                    'node_processes': ['namenode']}

        get_ng_templates.return_value = [r.NodeGroupTemplateResource(ngt_dict)]

        ct_dict = {'name': 'test', 'tenant_id': 't',
                   'plugin_name': 'fake', 'hadoop_version': '0.1'}

        get_cl_templates.return_value = [r.ClusterTemplateResource(ct_dict)]

    def _get_ng_template(id):
        for template in get_ng_templates():
            if template.id == id:
                return template
        return None

    if patch_templates:
        get_ng_template.side_effect = _get_ng_template
    # request data to validate
    patchers = [get_clusters_p, get_cluster_p,
                nova_p, get_image_p, heat_p, image_manager_p, cinder_p,
                cinder_exists_p, neutron_p]
    if patch_templates:
        patchers.extend([get_ng_template_p, get_ng_templates_p,
                         get_cl_template_p, get_cl_templates_p])
    return patchers


def stop_patch(patchers):
    for patcher in reversed(patchers):
        patcher.stop()


class ValidationTestCase(base.SaharaTestCase):
    def setUp(self):
        super(ValidationTestCase, self).setUp()
        self._create_object_fun = None
        self.scheme = None
        self.override_config('plugins', ['fake'])

    def tearDown(self):
        self._create_object_fun = None
        super(ValidationTestCase, self).tearDown()

    def _assert_calls(self, mock, call_info):
        if not call_info:
            self.assertEqual(0, mock.call_count, "Unexpected call to %s: %s"
                             % (mock.name, str(mock.call_args)))
        else:
            self.assertEqual(call_info[0], mock.call_count)
            self.assertEqual(call_info[1], mock.call_args[0][0].code)
            possible_messages = ([call_info[2]] if isinstance(
                call_info[2], six.string_types) else call_info[2])
            match = False
            check = mock.call_args[0][0].message
            if check.find('Error ID:') != -1:
                check = check.split('\n')[0]
            for message in possible_messages:
                if self._check_match(message, check):
                    match = True
                    break
            if not match:
                self.assertIn(check, possible_messages)

    def _check_match(self, expected, actual):
        d1, r1 = self._extract_printed_dict(expected)
        d2, r2 = self._extract_printed_dict(actual)

        # Note(slukjanov): regex needed because of different
        #                  versions of jsonschema generate different
        #                  messages.
        return (r1 == r2 or re.match(r1, r2)) and (d1 == d2)

    def _extract_printed_dict(self, s):
        start = s.find('{')
        if start == -1:
            return None, s

        end = s.rfind('}')
        if end == -1:
            return None, s

        return ast.literal_eval(s[start:end+1]), s[0:start+1] + s[end]

    @mock.patch("sahara.utils.api.request_data")
    @mock.patch("sahara.utils.api.bad_request")
    def _assert_create_object_validation(
            self, bad_req=None, request_data=None,
            data=None, bad_req_i=None):

        request_data.return_value = data
        # mock function that should be validated
        patchers = start_patch()
        m_func = mock.Mock()
        m_func.__name__ = "m_func"
        v.validate(self.scheme, self._create_object_fun)(m_func)(data=data)

        self.assertEqual(1, request_data.call_count)
        self._assert_calls(bad_req, bad_req_i)
        stop_patch(patchers)

    def _assert_valid_name_hostname_validation(self, data):

        data.update({'name': None})
        self._assert_create_object_validation(
            data=data,
            bad_req_i=(1, "VALIDATION_ERROR",
                       u"name: None is not of type 'string'")
        )
        data.update({'name': ""})
        self._assert_create_object_validation(
            data=data,
            bad_req_i=(1, "VALIDATION_ERROR",
                       u"name: '' is too short")
        )
        data.update({'name': ('a' * 51)})
        self._assert_create_object_validation(
            data=data,
            bad_req_i=(1, "VALIDATION_ERROR",
                       u"name: '%s' is too long" % ('a' * 51))
        )
        data.update({'name': 'a-!'})
        self._assert_create_object_validation(
            data=data,
            bad_req_i=(1, "VALIDATION_ERROR",
                       u"name: 'a-!' is not a 'valid_name_hostname'")
        )

    def _prop_types_str(self, prop_types):
        return ", ".join(["'%s'" % prop for prop in prop_types])

    def _assert_types(self, default_data):
        for p_name in self.scheme['properties']:
            prop = self.scheme['properties'][p_name]
            prop_types = prop["type"]
            if type(prop_types) is not list:
                prop_types = [prop_types]
            for prop_type in prop_types:
                if prop_type in _types_checks:
                    for type_ex in _types_checks[prop_type]:
                        data = default_data.copy()
                        value = type_ex
                        value_str = str(value)
                        if isinstance(value, str):
                            value_str = "'%s'" % value_str
                        data.update({p_name: value})
                        message = ("%s: %s is not of type %s" %
                                   (p_name, value_str,
                                    self._prop_types_str(prop_types)))
                        if "enum" in prop:
                            message = [message, "%s: %s is not one of %s" %
                                                (p_name, value_str,
                                                 prop["enum"])]
                        self._assert_create_object_validation(
                            data=data,
                            bad_req_i=(1, 'VALIDATION_ERROR', message)
                        )

    def _assert_cluster_configs_validation(self, require_image_id=False):
        data = {
            'name': 'test-cluster',
            'plugin_name': 'fake',
            'hadoop_version': '0.1',
            'cluster_configs': {
                'HDFS': {
                    'mapreduce.task.tmp.dir': '/temp/'
                }
            },
            'default_image_id': '550e8400-e29b-41d4-a716-446655440000'
        }
        if require_image_id:
            data_without_image = data.copy()
            data_without_image.pop('default_image_id')
            self._assert_create_object_validation(
                data=data_without_image,
                bad_req_i=(1, 'NOT_FOUND',
                           "'default_image_id' field is not found")
            )
        self._assert_create_object_validation(
            data=_update_data(data.copy(), {
                'cluster_configs': {
                    'wrong_target': {
                        'mapreduce.task.tmp.dir': '/temp/'
                    }
                }}),
            bad_req_i=(1, 'INVALID_REFERENCE',
                       "Plugin doesn't contain applicable "
                       "target 'wrong_target'")
        )
        self._assert_create_object_validation(
            data=_update_data(data.copy(), {
                'cluster_configs': {
                    'general': {
                        's': '/temp/'
                    }
                }
            }),
            bad_req_i=(1, 'INVALID_REFERENCE',
                       "Plugin's applicable target 'general' doesn't "
                       "contain config with name 's'")
        )

    def _assert_cluster_default_image_tags_validation(self):
        data = {
            'name': 'test-cluster',
            'plugin_name': 'fake',
            'hadoop_version': '0.1',
            'default_image_id': '550e8400-e29b-41d4-a716-446655440000',
            'domain_name': 'domain.org.',
            'neutron_management_network': 'd9a3bebc-f788-4b81-'
                                          '9a93-aa048022c1ca'
        }
        self._assert_create_object_validation(data=data)
        data = {
            'name': 'test-cluster',
            'plugin_name': 'fake',
            'hadoop_version': '0.1',
            'default_image_id': '813fe450-40d2-4acc-ade5-ea753a1bd5bc'
        }
        self._assert_create_object_validation(
            data=data,
            bad_req_i=(1, 'INVALID_REFERENCE',
                       "Requested image "
                       "'813fe450-40d2-4acc-ade5-ea753a1bd5bc' "
                       "doesn't contain required tags: "
                       "['0.1']"))

    def assert_protected_resource_exception(self, ex):
        self.assertIn("marked as protected", six.text_type(ex))

    def assert_created_in_another_tenant_exception(self, ex):
        self.assertIn("wasn't created in this tenant", six.text_type(ex))



# Copyright (c) 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from oslo_utils import uuidutils
import yaml

from sahara import exceptions as ex
from sahara.plugins import exceptions as p_ex
from sahara.plugins import images
from sahara.tests.unit import base as b


class TestImages(b.SaharaTestCase):

    def test_package_spec(self):
        cls = images.SaharaPackageValidator

        validator = cls.from_spec("java", {}, [])
        self.assertIsInstance(validator, cls)
        self.assertEqual(str(validator.packages[0]), "java")

        validator = cls.from_spec({"java": {"version": "8"}}, {}, [])
        self.assertIsInstance(validator, cls)
        self.assertEqual(str(validator.packages[0]), "java-8")

        validator = cls.from_spec(
            [{"java": {"version": "8"}}, "hadoop"], {}, [])
        self.assertIsInstance(validator, cls)
        self.assertEqual(str(validator.packages[0]), "java-8")
        self.assertEqual(str(validator.packages[1]), "hadoop")

    def test_script_spec(self):
        cls = images.SaharaScriptValidator
        resource_roots = ['tests/unit/plugins']

        validator = cls.from_spec('test_images.py', {}, resource_roots)
        self.assertIsInstance(validator, cls)
        self.assertEqual(validator.env_vars, ['test_only', 'distro'])

        validator = cls.from_spec(
            {'test_images.py': {'env_vars': ['extra-file', 'user']}},
            {}, resource_roots)
        self.assertIsInstance(validator, cls)
        self.assertEqual(validator.env_vars,
                         ['test_only', 'distro',
                          'extra-file', 'user'])

    def test_all_spec(self):
        cls = images.SaharaAllValidator
        validator_map = images.SaharaImageValidatorBase.get_validator_map()

        validator = cls.from_spec(
            [{'package': {'java': {'version': '8'}}}, {'package': 'hadoop'}],
            validator_map, [])
        self.assertIsInstance(validator, cls)
        self.assertEqual(len(validator.validators), 2)
        self.assertEqual(validator.validators[0].packages[0].name, 'java')
        self.assertEqual(validator.validators[1].packages[0].name, 'hadoop')

    def test_any_spec(self):
        cls = images.SaharaAnyValidator
        validator_map = images.SaharaImageValidatorBase.get_validator_map()

        validator = cls.from_spec(
            [{'package': {'java': {'version': '8'}}}, {'package': 'hadoop'}],
            validator_map, [])
        self.assertIsInstance(validator, cls)
        self.assertEqual(len(validator.validators), 2)
        self.assertEqual(validator.validators[0].packages[0].name, 'java')
        self.assertEqual(validator.validators[1].packages[0].name, 'hadoop')

    def test_os_case_spec(self):
        cls = images.SaharaOSCaseValidator
        validator_map = images.SaharaImageValidatorBase.get_validator_map()
        spec = [
            {'redhat': [{'package': 'nfs-utils'}]},
            {'debian': [{'package': 'nfs-common'}]}
        ]

        validator = cls.from_spec(spec, validator_map, [])
        self.assertIsInstance(validator, cls)
        self.assertEqual(len(validator.distros), 2)
        self.assertEqual(validator.distros[0].distro, 'redhat')
        self.assertEqual(validator.distros[1].distro, 'debian')
        redhat, debian = (
            validator.distros[os].validator.validators[0].packages[0].name
            for os in range(2))
        self.assertEqual(redhat, 'nfs-utils')
        self.assertEqual(debian, 'nfs-common')

    def test_sahara_image_validator_spec(self):
        cls = images.SaharaImageValidator
        validator_map = images.SaharaImageValidatorBase.get_validator_map()
        resource_roots = ['tests/unit/plugins']

        spec = """
            arguments:
              java-version:
                description: The version of java.
                default: openjdk
                required: false
                choices:
                  - openjdk
                  - oracle-java

            validators:
              - os_case:
                  - redhat:
                      - package: nfs-utils
                  - debian:
                      - package: nfs-common
              - any:
                - all:
                  - package: java-1.8.0-openjdk-devel
                  - argument_set:
                      argument_name: java-version
                      value: 1.8.0
                - all:
                  - package: java-1.7.0-openjdk-devel
                  - argument_set:
                      argument_name: java-version
                      value: 1.7.0
              - script: test_images.py
              - package:
                - hadoop
                - hadoop-libhdfs
                - hadoop-native
                - hadoop-pipes
                - hadoop-sbin
                - hadoop-lzo
                - lzo
                - lzo-devel
                - hadoop-lzo-native
              - argument_case:
                  argument_name: JAVA_VERSION
                  cases:
                    1.7.0:
                      - script: test_images.py
                    1.8.0:
                      - script: test_images.py

        """
        spec = yaml.safe_load(spec)

        validator = cls.from_spec(spec, validator_map, resource_roots)
        validators = validator.validators

        self.assertIsInstance(validator, cls)
        self.assertEqual(len(validators), 5)
        self.assertIsInstance(validators[0], images.SaharaOSCaseValidator)
        self.assertIsInstance(validators[1], images.SaharaAnyValidator)
        self.assertIsInstance(validators[2], images.SaharaScriptValidator)
        self.assertIsInstance(validators[3], images.SaharaPackageValidator)
        self.assertIsInstance(
            validators[4], images.SaharaArgumentCaseValidator)
        self.assertEqual(1, len(validator.arguments))
        self.assertEqual(validator.arguments['java-version'].required, False)
        self.assertEqual(validator.arguments['java-version'].default,
                         'openjdk')
        self.assertEqual(validator.arguments['java-version'].description,
                         'The version of java.')
        self.assertEqual(validator.arguments['java-version'].choices,
                         ['openjdk', 'oracle-java'])

    def test_package_validator_redhat(self):
        cls = images.SaharaPackageValidator
        image_arguments = {"distro": 'centos'}

        packages = [cls.Package("java", "8")]
        validator = images.SaharaPackageValidator(packages)
        remote = mock.Mock()
        validator.validate(remote, test_only=True,
                           image_arguments=image_arguments)
        remote.execute_command.assert_called_with(
            "rpm -q java-8", run_as_root=True)

        image_arguments = {"distro": 'fedora'}
        packages = [cls.Package("java", "8"), cls.Package("hadoop")]
        validator = images.SaharaPackageValidator(packages)
        remote = mock.Mock()
        remote.execute_command.side_effect = (
            ex.RemoteCommandException("So bad!"))
        try:
            validator.validate(remote, test_only=True,
                               image_arguments=image_arguments)
        except p_ex.ImageValidationError as e:
            self.assertIn("So bad!", e.message)
        remote.execute_command.assert_called_with(
            "rpm -q java-8 hadoop", run_as_root=True)
        self.assertEqual(remote.execute_command.call_count, 1)

        image_arguments = {"distro": 'redhat'}
        packages = [cls.Package("java", "8"), cls.Package("hadoop")]
        validator = images.SaharaPackageValidator(packages)
        remote = mock.Mock()

        def side_effect(call, run_as_root=False):
            if "rpm" in call:
                raise ex.RemoteCommandException("So bad!")

        remote.execute_command.side_effect = side_effect
        try:
            validator.validate(remote, test_only=False,
                               image_arguments=image_arguments)
        except p_ex.ImageValidationError as e:
            self.assertIn("So bad!", e.message)
        self.assertEqual(remote.execute_command.call_count, 3)
        calls = [mock.call("rpm -q java-8 hadoop", run_as_root=True),
                 mock.call("yum install -y java-8 hadoop", run_as_root=True),
                 mock.call("rpm -q java-8 hadoop", run_as_root=True)]
        remote.execute_command.assert_has_calls(calls)

    def test_package_validator_debian(self):
        cls = images.SaharaPackageValidator
        image_arguments = {"distro": 'ubuntu'}

        packages = [cls.Package("java", "8")]
        validator = images.SaharaPackageValidator(packages)
        remote = mock.Mock()
        validator.validate(remote, test_only=True,
                           image_arguments=image_arguments)
        remote.execute_command.assert_called_with(
            "dpkg -s java-8", run_as_root=True)

        image_arguments = {"distro": 'ubuntu'}
        packages = [cls.Package("java", "8"), cls.Package("hadoop")]
        validator = images.SaharaPackageValidator(packages)
        remote = mock.Mock()
        remote.execute_command.side_effect = (
            ex.RemoteCommandException("So bad!"))
        try:
            validator.validate(remote, test_only=True,
                               image_arguments=image_arguments)
        except p_ex.ImageValidationError as e:
            self.assertIn("So bad!", e.message)
        remote.execute_command.assert_called_with(
            "dpkg -s java-8 hadoop", run_as_root=True)
        self.assertEqual(remote.execute_command.call_count, 1)

        image_arguments = {"distro": 'ubuntu'}
        packages = [cls.Package("java", "8"), cls.Package("hadoop")]
        validator = images.SaharaPackageValidator(packages)
        remote = mock.Mock()
        remote.execute_command.side_effect = (
            ex.RemoteCommandException("So bad!"))
        try:
            validator.validate(remote, test_only=False,
                               image_arguments=image_arguments)
        except p_ex.ImageValidationError as e:
            self.assertIn("So bad!", e.message)
        self.assertEqual(remote.execute_command.call_count, 2)
        calls = [mock.call("dpkg -s java-8 hadoop", run_as_root=True),
                 mock.call("DEBIAN_FRONTEND=noninteractive " +
                           "apt-get -y install java-8 hadoop",
                           run_as_root=True)]
        remote.execute_command.assert_has_calls(calls)

    @mock.patch('oslo_utils.uuidutils.generate_uuid')
    def test_script_validator(self, uuid):
        hash_value = '00000000-0000-0000-0000-000000000000'
        uuidutils.generate_uuid.return_value = hash_value
        cls = images.SaharaScriptValidator
        image_arguments = {"distro": 'centos'}
        cmd = b"It's dangerous to go alone. Run this."
        validator = cls(cmd, env_vars=image_arguments.keys(),
                        output_var="distro")

        remote = mock.Mock(
            execute_command=mock.Mock(
                return_value=(0, 'fedora')))

        validator.validate(remote, test_only=False,
                           image_arguments=image_arguments)
        call = [mock.call('chmod +x /tmp/%(hash_value)s.sh' %
                          {'hash_value': hash_value}, run_as_root=True),
                mock.call('/tmp/%(hash_value)s.sh' %
                          {'hash_value': hash_value}, run_as_root=True)]
        remote.execute_command.assert_has_calls(call)
        self.assertEqual(image_arguments['distro'], 'fedora')

    def test_any_validator(self):
        cls = images.SaharaAnyValidator

        class FakeValidator(images.SaharaImageValidatorBase):

            def __init__(self, mock_validate):
                self.mock_validate = mock_validate

            def validate(self, remote, test_only=False, **kwargs):
                self.mock_validate(remote, test_only=test_only, **kwargs)

        # One success short circuits validation
        always_tells_the_truth = FakeValidator(mock.Mock())
        validator = cls([always_tells_the_truth, always_tells_the_truth])
        validator.validate(None, test_only=False)
        self.assertEqual(always_tells_the_truth.mock_validate.call_count, 1)

        # All failures fails, and calls with test_only=True on all first
        always_lies = FakeValidator(
            mock.Mock(side_effect=p_ex.ImageValidationError("Oh no!")))
        validator = cls([always_lies, always_lies])
        try:
            validator.validate(None, test_only=False)
        except p_ex.ImageValidationError:
            pass
        self.assertEqual(always_lies.mock_validate.call_count, 4)

        # But it fails after a first pass if test_only=True.
        always_lies = FakeValidator(
            mock.Mock(side_effect=p_ex.ImageValidationError("Oh no!")))
        validator = cls([always_lies, always_lies])
        try:
            validator.validate(None, test_only=True)
        except p_ex.ImageValidationError:
            pass
        self.assertEqual(always_lies.mock_validate.call_count, 2)

        # One failure doesn't end iteration.
        always_tells_the_truth = FakeValidator(mock.Mock())
        always_lies = FakeValidator(
            mock.Mock(side_effect=p_ex.ImageValidationError("Oh no!")))
        validator = cls([always_lies, always_tells_the_truth])
        validator.validate(None, test_only=False)
        self.assertEqual(always_lies.mock_validate.call_count, 1)
        self.assertEqual(always_tells_the_truth.mock_validate.call_count, 1)

    def test_all_validator(self):
        cls = images.SaharaAllValidator

        # All pass
        always_tells_the_truth = mock.Mock()
        validator = cls([always_tells_the_truth, always_tells_the_truth])
        validator.validate(None, test_only=False)
        self.assertEqual(always_tells_the_truth.validate.call_count, 2)
        always_tells_the_truth.validate.assert_called_with(
            None, test_only=False, image_arguments=None)

        # Second fails
        always_tells_the_truth = mock.Mock()
        always_lies = mock.Mock(validate=mock.Mock(
            side_effect=p_ex.ImageValidationError("Boom!")))
        validator = cls([always_tells_the_truth, always_lies])
        try:
            validator.validate(None, test_only=True)
        except p_ex.ImageValidationError:
            pass
        self.assertEqual(always_tells_the_truth.validate.call_count, 1)
        self.assertEqual(always_lies.validate.call_count, 1)
        always_tells_the_truth.validate.assert_called_with(
            None, test_only=True, image_arguments=None)
        always_lies.validate.assert_called_with(
            None, test_only=True, image_arguments=None)

        # First fails
        always_tells_the_truth = mock.Mock()
        always_lies = mock.Mock(validate=mock.Mock(
            side_effect=p_ex.ImageValidationError("Boom!")))
        validator = cls([always_lies, always_tells_the_truth])
        try:
            validator.validate(None, test_only=True, image_arguments={})
        except p_ex.ImageValidationError:
            pass
        self.assertEqual(always_lies.validate.call_count, 1)
        always_lies.validate.assert_called_with(
            None, test_only=True, image_arguments={})
        self.assertEqual(always_tells_the_truth.validate.call_count, 0)

    def test_os_case_validator(self):
        cls = images.SaharaOSCaseValidator
        Distro = images.SaharaOSCaseValidator._distro_tuple

        # First match wins and short circuits iteration
        centos = Distro("centos", mock.Mock())
        redhat = Distro("redhat", mock.Mock())
        distros = [centos, redhat]
        image_arguments = {images.SaharaImageValidator.DISTRO_KEY: "centos"}
        validator = cls(distros)
        validator.validate(None, test_only=False,
                           image_arguments=image_arguments)
        self.assertEqual(centos.validator.validate.call_count, 1)
        self.assertEqual(redhat.validator.validate.call_count, 0)
        centos.validator.validate.assert_called_with(
            None, test_only=False, image_arguments=image_arguments)

        # Families match
        centos = Distro("centos", mock.Mock())
        redhat = Distro("redhat", mock.Mock())
        distros = [centos, redhat]
        image_arguments = {images.SaharaImageValidator.DISTRO_KEY: "fedora"}
        validator = cls(distros)
        validator.validate(None, test_only=False,
                           image_arguments=image_arguments)
        self.assertEqual(centos.validator.validate.call_count, 0)
        self.assertEqual(redhat.validator.validate.call_count, 1)
        redhat.validator.validate.assert_called_with(
            None, test_only=False, image_arguments=image_arguments)

        # Non-matches do nothing
        centos = Distro("centos", mock.Mock())
        redhat = Distro("redhat", mock.Mock())
        distros = [centos, redhat]
        image_arguments = {images.SaharaImageValidator.DISTRO_KEY: "ubuntu"}
        validator = cls(distros)
        validator.validate(None, test_only=False,
                           image_arguments=image_arguments)
        self.assertEqual(centos.validator.validate.call_count, 0)
        self.assertEqual(redhat.validator.validate.call_count, 0)

    def test_sahara_argument_case_validator(self):
        cls = images.SaharaArgumentCaseValidator

        # Match gets called
        image_arguments = {"argument": "value"}
        match = mock.Mock()
        nomatch = mock.Mock()
        cases = {"value": match,
                 "another_value": nomatch}
        validator = cls("argument", cases)
        validator.validate(None, test_only=False,
                           image_arguments=image_arguments)
        self.assertEqual(match.validate.call_count, 1)
        self.assertEqual(nomatch.validate.call_count, 0)
        match.validate.assert_called_with(
            None, test_only=False, image_arguments=image_arguments)

        # Non-matches do nothing
        image_arguments = {"argument": "value"}
        nomatch = mock.Mock()
        cases = {"some_value": nomatch,
                 "another_value": nomatch}
        validator = cls("argument", cases)
        validator.validate(None, test_only=False,
                           image_arguments=image_arguments)
        self.assertEqual(nomatch.validate.call_count, 0)

    def test_sahara_argument_set_validator(self):
        cls = images.SaharaArgumentSetterValidator

        # Old variable is overwritten
        image_arguments = {"argument": "value"}
        validator = cls("argument", "new_value")
        validator.validate(None, test_only=False,
                           image_arguments=image_arguments)
        self.assertEqual(image_arguments["argument"], "new_value")

        # New variable is set
        image_arguments = {"argument": "value"}
        validator = cls("another_argument", "value")
        validator.validate(None, test_only=False,
                           image_arguments=image_arguments)
        self.assertEqual(image_arguments,
                         {"argument": "value", "another_argument": "value"})

    def test_sahara_image_validator(self):
        cls = images.SaharaImageValidator

        sub_validator = mock.Mock(validate=mock.Mock())
        remote = mock.Mock(get_os_distrib=mock.Mock(
            return_value="centos"))
        validator = cls(sub_validator, {})
        validator.validate(remote, test_only=False, image_arguments={})
        expected_map = {images.SaharaImageValidatorBase.DISTRO_KEY: "centos"}
        sub_validator.validate.assert_called_with(
            remote, test_only=False, image_arguments=expected_map)
        expected_map = {images.SaharaImageValidatorBase.DISTRO_KEY: "centos"}
        validator.validate(remote, test_only=True, image_arguments={})
        sub_validator.validate.assert_called_with(
            remote, test_only=True, image_arguments=expected_map)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg
import six

from sahara import conductor as c
from sahara import context
from sahara.service.castellan import utils as key_manager
from sahara.service.edp.oozie.workflow_creator import hive_workflow
from sahara.service.edp.oozie.workflow_creator import java_workflow
from sahara.service.edp.oozie.workflow_creator import mapreduce_workflow
from sahara.service.edp.oozie.workflow_creator import pig_workflow
from sahara.service.edp.oozie.workflow_creator import shell_workflow
from sahara.service.edp import s3_common
from sahara.swift import swift_helper as sw
from sahara.swift import utils as su
from sahara.utils import edp
from sahara.utils import xmlutils


conductor = c.API
CONF = cfg.CONF


class BaseFactory(object):
    def _separate_edp_configs(self, job_dict):
        configs = {}
        edp_configs = {}
        if 'configs' in job_dict:
            for k, v in six.iteritems(job_dict['configs']):
                if k.startswith('edp.'):
                    edp_configs[k] = v
                elif not k.startswith('oozie.'):
                    # 'oozie.' configs have been written to the properties file
                    configs[k] = v
        return configs, edp_configs

    def _prune_edp_configs(self, job_dict):
        if job_dict is None:
            return {}, {}

        # Rather than copy.copy, we make this by hand
        # to avoid FrozenClassError when we update 'configs'
        pruned_job_dict = {}
        for k, v in six.iteritems(job_dict):
            pruned_job_dict[k] = v

        # Separate out "edp." configs into its own dictionary
        configs, edp_configs = self._separate_edp_configs(job_dict)

        # Prune the new job_dict so it does not hold "edp." configs
        pruned_job_dict['configs'] = configs

        return pruned_job_dict, edp_configs

    def _update_dict(self, dest, src):
        if src is not None:
            for key, value in six.iteritems(dest):
                if hasattr(value, "update"):
                    new_vals = src.get(key, {})
                    value.update(new_vals)

    def update_job_dict(self, job_dict, exec_dict):
        pruned_exec_dict, edp_configs = self._prune_edp_configs(exec_dict)
        self._update_dict(job_dict, pruned_exec_dict)

        # Add the separated "edp." configs to the job_dict
        job_dict['edp_configs'] = edp_configs

        # Args are listed, not named. Simply replace them.
        job_dict['args'] = pruned_exec_dict.get('args', [])

        # Find all swift:// paths in args, configs, and params and
        # add the .sahara suffix to the container if it is not there
        # already
        job_dict['args'] = [
            # TODO(tmckay) args for Pig can actually be -param name=value
            # and value could conceivably contain swift paths
            su.inject_swift_url_suffix(arg) for arg in job_dict['args']]

        for k, v in six.iteritems(job_dict.get('configs', {})):
            job_dict['configs'][k] = su.inject_swift_url_suffix(v)

        for k, v in six.iteritems(job_dict.get('params', {})):
            job_dict['params'][k] = su.inject_swift_url_suffix(v)

    def get_configs(self, input_data, output_data, proxy_configs=None):
        configs = {}

        if proxy_configs:
            configs[sw.HADOOP_SWIFT_USERNAME] = proxy_configs.get(
                'proxy_username')
            configs[sw.HADOOP_SWIFT_PASSWORD] = key_manager.get_secret(
                proxy_configs.get('proxy_password'))
            configs[sw.HADOOP_SWIFT_TRUST_ID] = proxy_configs.get(
                'proxy_trust_id')
            configs[sw.HADOOP_SWIFT_DOMAIN_NAME] = CONF.proxy_user_domain_name
            return configs

        for src in (input_data, output_data):
            if src.type == "swift" and hasattr(src, "credentials"):
                if "user" in src.credentials:
                    configs[sw.HADOOP_SWIFT_USERNAME] = src.credentials['user']
                if "password" in src.credentials:
                    configs[sw.HADOOP_SWIFT_PASSWORD] = (
                        key_manager.get_secret(src.credentials['password']))
                break
        for src in (input_data, output_data):
            if src.type == "s3" and hasattr(src, "credentials"):
                if "accesskey" in src.credentials:
                    configs[s3_common.S3_ACCESS_KEY_CONFIG] = (
                        src.credentials['accesskey'])
                if "secretkey" in src.credentials:
                    configs[s3_common.S3_SECRET_KEY_CONFIG] = (
                        key_manager.get_secret(src.credentials['secretkey']))
                if "endpoint" in src.credentials:
                    configs[s3_common.S3_ENDPOINT_CONFIG] = (
                        src.credentials['endpoint'])
                if "bucket_in_path" in src.credentials:
                    configs[s3_common.S3_BUCKET_IN_PATH_CONFIG] = (
                        src.credentials['bucket_in_path'])
                if "ssl" in src.credentials:
                    configs[s3_common.S3_SSL_CONFIG] = (
                        src.credentials['ssl'])
                break
        return configs

    def get_params(self, input_data, output_data, data_source_urls):
        return {'INPUT': data_source_urls[input_data.id],
                'OUTPUT': data_source_urls[output_data.id]}


class PigFactory(BaseFactory):
    def __init__(self, job):
        super(PigFactory, self).__init__()

        self.name = self.get_script_name(job)

    def get_script_name(self, job):
        return conductor.job_main_name(context.ctx(), job)

    def get_workflow_xml(self, cluster, job_configs, input_data, output_data,
                         hdfs_user, data_source_urls):
        proxy_configs = job_configs.get('proxy_configs')
        job_dict = {'configs': self.get_configs(input_data, output_data,
                                                proxy_configs),
                    'params': self.get_params(input_data, output_data,
                                              data_source_urls),
                    'args': []}
        self.update_job_dict(job_dict, job_configs)
        creator = pig_workflow.PigWorkflowCreator()
        creator.build_workflow_xml(self.name,
                                   configuration=job_dict['configs'],
                                   params=job_dict['params'],
                                   arguments=job_dict['args'])
        return creator.get_built_workflow_xml()


class HiveFactory(BaseFactory):
    def __init__(self, job):
        super(HiveFactory, self).__init__()

        self.name = self.get_script_name(job)

    def get_script_name(self, job):
        return conductor.job_main_name(context.ctx(), job)

    def get_workflow_xml(self, cluster, job_configs, input_data, output_data,
                         hdfs_user, data_source_urls):
        proxy_configs = job_configs.get('proxy_configs')
        job_dict = {'configs': self.get_configs(input_data, output_data,
                                                proxy_configs),
                    'params': self.get_params(input_data, output_data,
                                              data_source_urls)}
        self.update_job_dict(job_dict, job_configs)

        creator = hive_workflow.HiveWorkflowCreator()
        creator.build_workflow_xml(self.name,
                                   edp.get_hive_shared_conf_path(hdfs_user),
                                   configuration=job_dict['configs'],
                                   params=job_dict['params'])
        return creator.get_built_workflow_xml()


class MapReduceFactory(BaseFactory):

    def get_configs(self, input_data, output_data, proxy_configs,
                    data_source_urls):
        configs = super(MapReduceFactory, self).get_configs(input_data,
                                                            output_data,
                                                            proxy_configs)
        configs['mapred.input.dir'] = data_source_urls[input_data.id]
        configs['mapred.output.dir'] = data_source_urls[output_data.id]
        return configs

    def _get_streaming(self, job_dict):
        prefix = 'edp.streaming.'
        return {k[len(prefix):]: v for (k, v) in six.iteritems(
            job_dict['edp_configs']) if k.startswith(prefix)}

    def get_workflow_xml(self, cluster, job_configs, input_data, output_data,
                         hdfs_user, data_source_urls):
        proxy_configs = job_configs.get('proxy_configs')
        job_dict = {'configs': self.get_configs(input_data, output_data,
                                                proxy_configs,
                                                data_source_urls)}
        self.update_job_dict(job_dict, job_configs)
        creator = mapreduce_workflow.MapReduceWorkFlowCreator()
        creator.build_workflow_xml(configuration=job_dict['configs'],
                                   streaming=self._get_streaming(job_dict))
        return creator.get_built_workflow_xml()


class JavaFactory(BaseFactory):

    def _get_java_configs(self, job_dict):
        main_class = job_dict['edp_configs']['edp.java.main_class']
        java_opts = job_dict['edp_configs'].get('edp.java.java_opts', None)
        args = job_dict['args']
        if edp.is_adapt_for_oozie_enabled(job_dict['edp_configs']):
            if args:
                args = [main_class] + args
            else:
                args = [main_class]
            main_class = 'org.openstack.sahara.edp.MainWrapper'
        return main_class, java_opts, args

    def get_configs(self, proxy_configs=None):
        # TODO(jfreud): allow s3 and non-proxy swift configs here?
        configs = {}

        if proxy_configs:
            configs[sw.HADOOP_SWIFT_USERNAME] = proxy_configs.get(
                'proxy_username')
            configs[sw.HADOOP_SWIFT_PASSWORD] = key_manager.get_secret(
                proxy_configs.get('proxy_password'))
            configs[sw.HADOOP_SWIFT_TRUST_ID] = proxy_configs.get(
                'proxy_trust_id')
            configs[sw.HADOOP_SWIFT_DOMAIN_NAME] = CONF.proxy_user_domain_name
            return configs

        return configs

    def get_workflow_xml(self, cluster, job_configs, *args, **kwargs):
        proxy_configs = job_configs.get('proxy_configs')
        job_dict = {'configs': self.get_configs(proxy_configs=proxy_configs),
                    'args': []}
        self.update_job_dict(job_dict, job_configs)

        main_class, java_opts, args = self._get_java_configs(job_dict)
        creator = java_workflow.JavaWorkflowCreator()
        creator.build_workflow_xml(main_class,
                                   configuration=job_dict['configs'],
                                   java_opts=java_opts,
                                   arguments=args)
        return creator.get_built_workflow_xml()


class ShellFactory(BaseFactory):

    def __init__(self, job):
        self.name, self.file_names = self.get_file_names(job)

    def get_file_names(self, job):
        ctx = context.ctx()
        return (conductor.job_main_name(ctx, job),
                conductor.job_lib_names(ctx, job))

    def get_configs(self):
        return {'configs': {},
                'params': {},
                'args': []}

    def get_workflow_xml(self, cluster, job_configs, *args, **kwargs):
        job_dict = self.get_configs()
        self.update_job_dict(job_dict, job_configs)
        creator = shell_workflow.ShellWorkflowCreator()
        creator.build_workflow_xml(self.name,
                                   configuration=job_dict['configs'],
                                   env_vars=job_dict['params'],
                                   arguments=job_dict['args'],
                                   files=self.file_names)
        return creator.get_built_workflow_xml()


def _get_creator(job):

    def make_PigFactory():
        return PigFactory(job)

    def make_HiveFactory():
        return HiveFactory(job)

    def make_ShellFactory():
        return ShellFactory(job)

    type_map = {
        edp.JOB_TYPE_HIVE: make_HiveFactory,
        edp.JOB_TYPE_JAVA: JavaFactory,
        edp.JOB_TYPE_MAPREDUCE: MapReduceFactory,
        edp.JOB_TYPE_MAPREDUCE_STREAMING: MapReduceFactory,
        edp.JOB_TYPE_PIG: make_PigFactory,
        edp.JOB_TYPE_SHELL: make_ShellFactory
    }

    return type_map[job.type]()


def get_workflow_xml(job, cluster, job_configs, *args, **kwargs):
    return _get_creator(job).get_workflow_xml(
        cluster, job_configs, *args, **kwargs)


def get_possible_job_config(job_type):
    if not edp.compare_job_type(job_type, *edp.JOB_TYPES_ALL):
        return None

    if edp.compare_job_type(job_type, edp.JOB_TYPE_JAVA):
        return {'job_config': {'configs': [], 'args': []}}

    if edp.compare_job_type(job_type, edp.JOB_TYPE_SHELL):
        return {'job_config': {'configs': [], 'params': {}, 'args': []}}

    if edp.compare_job_type(job_type,
                            edp.JOB_TYPE_MAPREDUCE, edp.JOB_TYPE_PIG):
        cfg = xmlutils.load_hadoop_xml_defaults(
            'service/edp/resources/mapred-default.xml')
        if edp.compare_job_type(job_type, edp.JOB_TYPE_MAPREDUCE):
            cfg += get_possible_mapreduce_configs()
    elif edp.compare_job_type(job_type, edp.JOB_TYPE_HIVE):
        cfg = xmlutils.load_hadoop_xml_defaults(
            'service/edp/resources/hive-default.xml')

    config = {'configs': cfg}
    if edp.compare_job_type(job_type, edp.JOB_TYPE_PIG, edp.JOB_TYPE_HIVE):
        config.update({'params': {}})
    if edp.compare_job_type(job_type, edp.JOB_TYPE_PIG):
        config.update({'args': []})
    return {'job_config': config}


def get_possible_mapreduce_configs():
    '''return a list of possible configuration values for map reduce jobs.'''
    cfg = xmlutils.load_hadoop_xml_defaults(
        'service/edp/resources/mapred-job-config.xml')
    return cfg



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import testtools

from sahara.service.edp.oozie.workflow_creator import workflow_factory as w_f
from sahara.utils import edp


class TestJobPossibleConfigs(testtools.TestCase):

    def test_possible_configs(self):
        res = w_f.get_possible_job_config(edp.JOB_TYPE_MAPREDUCE)
        sample_config_property = {
            'name': 'mapreduce.jobtracker.expire.trackers.interval',
            'value': '600000',
            'description': "Expert: The time-interval, in miliseconds, after "
                           "whicha tasktracker is declared 'lost' if it "
                           "doesn't send heartbeats."
        }
        self.assertIn(sample_config_property, res['job_config']["configs"])

        res = w_f.get_possible_job_config(edp.JOB_TYPE_HIVE)
        sample_config_property = {
            "description": "The serde used by FetchTask to serialize the "
                           "fetch output.",
            "name": "hive.fetch.output.serde",
            "value": "org.apache.hadoop.hive.serde2.DelimitedJSONSerDe"
        }
        self.assertIn(sample_config_property, res["job_config"]['configs'])
        res = w_f.get_possible_job_config("impossible_config")
        self.assertIsNone(res)



# Copyright (c) 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

import sahara.service.edp.binary_retrievers.manila_share as ms
from sahara.tests.unit import base


class _FakeShare(object):
    def __init__(self, id, share_proto='NFS'):
        self.id = id
        self.share_proto = share_proto


class TestManilaShare(base.SaharaTestCase):
    def setUp(self):
        super(TestManilaShare, self).setUp()

    @mock.patch('sahara.utils.openstack.manila.client')
    @mock.patch('sahara.conductor.API.cluster_update')
    @mock.patch('sahara.service.edp.utils.shares.mount_shares')
    def test_get_file_info(self, mount_shares, cluster_update, f_manilaclient):
        cluster_shares = [
            {'id': 'the_share_id',
             'path': '/mnt/mymountpoint'}
        ]

        ng_shares = [
            {'id': 'the_share_id',
             'path': '/mnt/othermountpoint'},
            {'id': '123456',
             'path': '/mnt/themountpoint'}
        ]

        job_binary = mock.Mock()
        job_binary.url = 'manila://the_share_id/the_path'

        remote = mock.Mock()
        remote.instance.node_group.cluster.shares = cluster_shares
        remote.instance.node_group.shares = ng_shares

        info = ms.get_file_info(job_binary, remote)
        self.assertCountEqual({'path': '/mnt/mymountpoint/the_path',
                               'type': 'path'}, info)
        self.assertEqual(0, mount_shares.call_count)
        self.assertEqual(0, cluster_update.call_count)

        job_binary.url = 'manila://123456/the_path'
        info = ms.get_file_info(job_binary, remote)
        self.assertCountEqual({'path': '/mnt/themountpoint/the_path',
                               'type': 'path'}, info)
        self.assertEqual(0, mount_shares.call_count)
        self.assertEqual(0, cluster_update.call_count)

        # This should return a default path, and should cause
        # a mount at the default location
        share = _FakeShare("missing_id")
        f_manilaclient.return_value = mock.Mock(
            shares=mock.Mock(
                get=mock.Mock(return_value=share)))

        job_binary.url = 'manila://missing_id/the_path'
        info = ms.get_file_info(job_binary, remote)
        self.assertCountEqual({'path': '/mnt/missing_id/the_path',
                               'type': 'path'}, info)
        self.assertEqual(1, mount_shares.call_count)
        self.assertEqual(1, cluster_update.call_count)



# Copyright (c) 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from sahara.service import trusts
from sahara.tests.unit import base


class FakeTrust(object):

    def __init__(self, id):
        self.id = id


class TestTrusts(base.SaharaTestCase):

    def _client(self):
        create = mock.Mock()
        create.return_value = FakeTrust("trust_id")
        client_trusts = mock.Mock(create=create)
        client = mock.Mock(trusts=client_trusts)
        return client

    @mock.patch('sahara.utils.openstack.keystone.client_from_auth')
    @mock.patch('sahara.utils.openstack.keystone.project_id_from_auth')
    @mock.patch('sahara.utils.openstack.keystone.user_id_from_auth')
    def test_create_trust(self, user_id_from_auth, project_id_from_auth,
                          client_from_auth):
        project_id_from_auth.return_value = 'tenant_id'
        user_id_from_auth.side_effect = ['trustor_id', 'trustee_id']
        trustor = 'trustor_id'
        trustee = 'trustee_id'
        client = self._client()
        client_from_auth.return_value = client
        trust_id = trusts.create_trust(trustor, trustee,
                                       "role_names")
        client.trusts.create.assert_called_with(
            trustor_user="trustor_id",
            trustee_user="trustee_id",
            impersonation=True,
            role_names="role_names",
            project="tenant_id",
            allow_redelegation=False,
        )
        self.assertEqual("trust_id", trust_id)

        user_id_from_auth.side_effect = ['trustor_id', 'trustee_id']
        client = self._client()
        client_from_auth.return_value = client
        trust_id = trusts.create_trust(trustor, trustee, "role_names",
                                       project_id='injected_project')
        client.trusts.create.assert_called_with(trustor_user="trustor_id",
                                                trustee_user="trustee_id",
                                                impersonation=True,
                                                role_names="role_names",
                                                project="injected_project",
                                                allow_redelegation=False)
        self.assertEqual("trust_id", trust_id)

    @mock.patch('sahara.conductor.API.cluster_get')
    @mock.patch('sahara.conductor.API.cluster_update')
    @mock.patch('sahara.service.trusts.create_trust')
    @mock.patch('sahara.utils.openstack.keystone.auth_for_admin')
    @mock.patch('sahara.context.current')
    def test_create_trust_for_cluster(self, context_current, auth_for_admin,
                                      create_trust, cluster_update, cl_get):
        self.override_config('project_name', 'admin_project', group='trustee')
        trustor_auth = mock.Mock()
        fake_cluster = mock.Mock(trust_id=None)
        cl_get.return_value = fake_cluster
        ctx = mock.Mock(roles="role_names", auth_plugin=trustor_auth)
        context_current.return_value = ctx
        trustee_auth = mock.Mock()
        auth_for_admin.return_value = trustee_auth
        create_trust.return_value = 'trust_id'

        trusts.create_trust_for_cluster("cluster")

        auth_for_admin.assert_called_with(project_name='admin_project')
        create_trust.assert_called_with(trustor=trustor_auth,
                                        trustee=trustee_auth,
                                        role_names='role_names',
                                        allow_redelegation=True)

        cluster_update.assert_called_with(ctx, fake_cluster,
                                          {"trust_id": "trust_id"})

    @mock.patch('sahara.utils.openstack.keystone.client_from_auth')
    @mock.patch('sahara.utils.openstack.keystone.auth_for_admin')
    @mock.patch('sahara.service.trusts.create_trust')
    def test_delete_trust(self, trust, auth_for_admin,
                          client_from_auth):
        client = self._client()
        client_from_auth.return_value = client
        trust.return_value = 'test_id'
        trustor_auth = mock.Mock()
        trustee_auth = mock.Mock()
        auth_for_admin.return_value = trustee_auth
        trust_id = trusts.create_trust(trustor_auth, trustee_auth,
                                       "role_names")

        trusts.delete_trust(trustee_auth, trust_id)
        client.trusts.delete.assert_called_with(trust_id)

    @mock.patch('sahara.conductor.API.cluster_update')
    @mock.patch('sahara.utils.openstack.keystone.auth_for_admin')
    @mock.patch('sahara.service.trusts.delete_trust')
    @mock.patch('sahara.conductor.API.cluster_get')
    @mock.patch('sahara.context.current')
    def test_delete_trust_from_cluster(self, context_current, cl_get,
                                       delete_trust, auth_for_admin,
                                       cluster_update):
        fake_cluster = mock.Mock(trust_id='test_id')
        cl_get.return_value = fake_cluster
        trustor_auth = mock.Mock()
        trustee_auth = mock.Mock()
        auth_for_admin.return_value = trustee_auth
        ctx = mock.Mock(roles="role_names", auth_plugin=trustor_auth)
        context_current.return_value = ctx
        trusts.delete_trust_from_cluster("cluster")

        delete_trust.assert_called_with(trustee_auth, 'test_id')
        cluster_update.assert_called_with(ctx, fake_cluster,
                                          {"trust_id": None})



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from sahara.plugins import exceptions as ex
from sahara.plugins import utils as pu
from sahara.tests.unit import base as b


class FakeInstance(object):
    def __init__(self, _id, node_processes=None):
        self.id = _id
        self.node_processes = node_processes or []

    @property
    def node_group(self):
        return self

    def __eq__(self, other):
        return self.id == other.id


class FakeNodeGroup(object):

    def __init__(self, node_processes, instances=None):
        self.node_processes = node_processes
        self.instances = instances or []
        self.count = len(self.instances)

    def __eq__(self, other):
        return self.node_processes == other.node_processes


class TestPluginUtils(b.SaharaTestCase):

    def setUp(self):
        super(TestPluginUtils, self).setUp()
        self.cluster = mock.Mock()
        self.cluster.node_groups = [
            FakeNodeGroup(["node_process1"], [FakeInstance("1")]),
            FakeNodeGroup(["node_process2"], [FakeInstance("2")]),
            FakeNodeGroup(["node_process3"], [FakeInstance("3")]),
        ]

    def test_get_node_groups(self):
        res = pu.get_node_groups(self.cluster)
        self.assertEqual([
            FakeNodeGroup(["node_process1"]),
            FakeNodeGroup(["node_process2"]),
            FakeNodeGroup(["node_process3"]),
        ], res)

        res = pu.get_node_groups(self.cluster, "node_process1")
        self.assertEqual([
            FakeNodeGroup(["node_process1"])
        ], res)

        res = pu.get_node_groups(self.cluster, "node_process")
        self.assertEqual([], res)

    def test_get_instances_count(self):
        res = pu.get_instances_count(self.cluster)
        self.assertEqual(3, res)

        res = pu.get_instances_count(self.cluster, "node_process1")
        self.assertEqual(1, res)

    def test_get_instances(self):
        res = pu.get_instances(self.cluster)
        self.assertEqual([
            FakeInstance("1"), FakeInstance("2"), FakeInstance("3")], res)

        res = pu.get_instances(self.cluster, "node_process1")
        self.assertEqual([FakeInstance("1")], res)

    def test_get_instance(self):
        self.assertRaises(ex.InvalidComponentCountException,
                          pu.get_instance, self.cluster, None)

        res = pu.get_instance(self.cluster, "node_process")
        self.assertIsNone(res)

        res = pu.get_instance(self.cluster, "node_process1")
        self.assertEqual(FakeInstance("1"), res)

    def test_generate_host_names(self):
        node = mock.Mock()
        node.hostname = mock.Mock(return_value="host_name")

        res = pu.generate_host_names([node, node])
        self.assertEqual("host_name\nhost_name", res)

    def test_generate_fqdn_host_names(self):
        node = mock.Mock()
        node.fqdn = mock.Mock(return_value="fqdn")

        res = pu.generate_fqdn_host_names([node, node])
        self.assertEqual("fqdn\nfqdn", res)

    def test_get_port_from_address(self):

        res = pu.get_port_from_address("0.0.0.0:8000")
        self.assertEqual(8000, res)

        res = pu.get_port_from_address("http://localhost:8000/resource")
        self.assertEqual(8000, res)

        res = pu.get_port_from_address("http://192.168.1.101:10000")
        self.assertEqual(10000, res)

        res = pu.get_port_from_address("mydomain")
        self.assertIsNone(res)

    def test_instances_with_services(self):
        inst = [FakeInstance("1", ["nodeprocess1"]),
                FakeInstance("2", ["nodeprocess2"])]

        node_processes = ["nodeprocess"]
        res = pu.instances_with_services(inst, node_processes)
        self.assertEqual([], res)

        node_processes = ["nodeprocess1"]
        res = pu.instances_with_services(inst, node_processes)
        self.assertEqual([FakeInstance("1", ["nodeprocess1"])], res)

    @mock.patch("sahara.plugins.utils.plugins_base")
    def test_get_config_value_or_default(self, mock_plugins_base):
        # no config
        self.assertRaises(RuntimeError,
                          pu.get_config_value_or_default)

        config = mock.Mock()
        config.applicable_target = "service"
        config.name = "name"
        config.default_value = "default_value"

        # cluster has the config
        cluster = mock.Mock()
        cluster.cluster_configs = {"service": {"name": "name"}}
        cluster.plugin_name = "plugin_name"
        cluster.hadoop_version = "hadoop_version"

        res = pu.get_config_value_or_default(cluster=cluster,
                                             config=config)
        self.assertEqual("name", res)

        # node group has the config
        cluster.cluster_configs = {}

        node_group1 = mock.Mock()
        node_group2 = mock.Mock()

        node_group1.configuration = mock.Mock(return_value={"service": {}})

        node_group2.configuration = mock.Mock(
            return_value={"service": {"name": "name"}})

        cluster.node_groups = [node_group1, node_group2]

        res = pu.get_config_value_or_default(cluster=cluster,
                                             config=config)
        self.assertEqual("name", res)

        # cluster doesn't have the config, neither the node groups
        # so it returns the default value
        cluster.node_groups = []
        res = pu.get_config_value_or_default(cluster=cluster,
                                             config=config)
        self.assertEqual("default_value", res)

        # no config specified, but there's a config for the plugin
        # with this service and name
        mock_get_all_configs = mock.Mock(return_value=[config])
        mock_plugin = mock.Mock()
        mock_plugin.get_all_configs = mock_get_all_configs
        mock_get_plugin = mock.Mock(return_value=mock_plugin)
        mock_PLUGINS = mock.Mock()
        mock_PLUGINS.get_plugin = mock_get_plugin
        mock_plugins_base.PLUGINS = mock_PLUGINS

        res = pu.get_config_value_or_default(cluster=cluster,
                                             service="service", name="name")
        self.assertEqual("default_value", res)

        mock_get_plugin.assert_called_once_with("plugin_name")
        mock_get_all_configs.assert_called_once_with("hadoop_version")

        # no config especified and no existing config for this plugin
        # with this service or name
        cluster.plugin_name = "plugin_name2"
        cluster.hadoop_version = "hadoop_version2"
        self.assertRaises(RuntimeError,
                          pu.get_config_value_or_default, cluster=cluster,
                          service="newService", name="name")

        mock_get_plugin.assert_called_with("plugin_name2")
        self.assertEqual(2, mock_get_plugin.call_count)

        mock_get_all_configs.assert_called_with("hadoop_version2")
        self.assertEqual(2, mock_get_all_configs.call_count)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from sahara.service import networks
from sahara.tests.unit import base


class TestNetworks(base.SaharaTestCase):

    @mock.patch('sahara.service.networks.conductor.instance_update')
    @mock.patch('sahara.utils.openstack.nova.get_instance_info')
    def test_init_instances_ips_with_floating(self, nova, upd):
        server = mock.Mock()
        server.addresses = {
            'network': [
                {
                    'version': 4,
                    'OS-EXT-IPS:type': 'fixed',
                    'addr': '10.2.2.2'
                },
                {
                    'version': 4,
                    'OS-EXT-IPS:type': 'floating',
                    'addr': '172.1.1.1'
                }
            ]
        }
        nova.return_value = server

        self.assertEqual('172.1.1.1', networks.init_instances_ips(mock.Mock()))

    @mock.patch('sahara.service.networks.conductor.instance_update')
    @mock.patch('sahara.utils.openstack.nova.get_instance_info')
    def test_init_instances_ips_without_floating(self, nova, upd):
        self.override_config('use_floating_ips', False)
        server = mock.Mock()
        server.addresses = {
            'network': [
                {
                    'version': 4,
                    'OS-EXT-IPS:type': 'fixed',
                    'addr': '10.2.2.2'
                }
            ]
        }
        nova.return_value = server

        self.assertEqual('10.2.2.2', networks.init_instances_ips(mock.Mock()))

    @mock.patch('sahara.service.networks.conductor.instance_update')
    @mock.patch('sahara.utils.openstack.nova.get_instance_info')
    def test_init_instances_ips_with_proxy(self, nova, upd):

        instance = mock.Mock()
        instance.cluster.has_proxy_gateway.return_value = True
        instance.node_group.is_proxy_gateway = False
        server = mock.Mock()
        server.addresses = {
            'network': [
                {
                    'version': 4,
                    'OS-EXT-IPS:type': 'fixed',
                    'addr': '10.2.2.2'
                }
            ]
        }
        nova.return_value = server

        self.assertEqual('10.2.2.2', networks.init_instances_ips(instance))

    @mock.patch('sahara.service.networks.conductor.instance_update')
    @mock.patch('sahara.utils.openstack.nova.get_instance_info')
    def test_init_instances_ips_neutron_with_floating(
            self, nova, upd):

        server = mock.Mock(id='serv_id')
        server.addresses = {
            'network': [
                {
                    'version': 4,
                    'OS-EXT-IPS:type': 'floating',
                    'addr': '172.1.1.1'
                },
                {
                    'version': 4,
                    'OS-EXT-IPS:type': 'fixed',
                    'addr': '10.2.2.2'
                }
            ]
        }
        nova.return_value = server
        self.assertEqual('172.1.1.1', networks.init_instances_ips(mock.Mock()))

    @mock.patch('sahara.service.networks.conductor.instance_update')
    @mock.patch('sahara.utils.openstack.nova.get_instance_info')
    def test_init_instances_ips_neutron_without_floating(
            self, nova, upd):

        self.override_config('use_floating_ips', False)
        server = mock.Mock(id='serv_id')
        server.addresses = {
            'network': [
                {
                    'version': 4,
                    'OS-EXT-IPS:type': 'fixed',
                    'addr': '10.2.2.2'
                }
            ]
        }
        nova.return_value = server
        self.assertEqual('10.2.2.2', networks.init_instances_ips(mock.Mock()))

    @mock.patch('sahara.service.networks.conductor.instance_update')
    @mock.patch('sahara.utils.openstack.nova.get_instance_info')
    def test_init_instances_ips_with_ipv6_subnet(self, nova, upd):
        self.override_config('use_floating_ips', False)
        instance = mock.Mock()
        server = mock.Mock()
        server.addresses = {
            'network': [
                {
                    'version': 6,
                    'OS-EXT-IPS:type': 'fixed',
                    'addr': 'fe80::1234:5678:9abc:def0'
                },
                {
                    'version': 4,
                    'OS-EXT-IPS:type': 'fixed',
                    'addr': '10.2.2.2'
                }
            ]
        }
        nova.return_value = server

        self.assertEqual('10.2.2.2', networks.init_instances_ips(instance))



# Copyright (c) 2014 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from unittest import mock

from oslo_utils import uuidutils

from sahara.service.edp import job_utils
from sahara.tests.unit import base
from sahara.utils import proxy as p


class TestProxyUtils(base.SaharaWithDbTestCase):
    def setUp(self):
        super(TestProxyUtils, self).setUp()

    @mock.patch('sahara.service.castellan.utils.store_secret')
    @mock.patch('sahara.context.ctx')
    @mock.patch('sahara.conductor.API.job_execution_update')
    @mock.patch('sahara.service.trusts.create_trust')
    @mock.patch('sahara.utils.openstack.keystone.auth_for_proxy')
    @mock.patch('sahara.utils.openstack.keystone.auth')
    @mock.patch('sahara.utils.proxy.proxy_user_create')
    def test_create_proxy_user_for_job_execution(self, proxy_user, trustor,
                                                 trustee, trust,
                                                 job_execution_update,
                                                 context_current, passwd):
        job_execution = mock.Mock(id=1,
                                  output_id=2,
                                  job_id=3,
                                  job_configs=None)
        job_execution.job_configs = mock.Mock(to_dict=mock.Mock(
            return_value={}
        ))
        proxy_user.return_value = "proxy_user"
        passwd.return_value = "test_password"
        trustor.return_value = "test_trustor"
        trustee.return_value = "test_trustee"
        trust.return_value = "123456"
        ctx = mock.Mock()
        context_current.return_value = ctx
        p.create_proxy_user_for_job_execution(job_execution)
        update = {'job_configs': {'proxy_configs': None}}
        update['job_configs']['proxy_configs'] = {
            'proxy_username': 'job_1',
            'proxy_password': 'test_password',
            'proxy_trust_id': '123456'
        }
        job_execution_update.assert_called_with(ctx, job_execution, update)

    @mock.patch('sahara.conductor.API.job_get')
    @mock.patch('sahara.conductor.API.data_source_get')
    @mock.patch('sahara.conductor.API.data_source_count')
    @mock.patch('sahara.context.ctx')
    def test_job_execution_requires_proxy_user(self,
                                               ctx,
                                               data_source_count,
                                               data_source,
                                               job):

        self.override_config('use_domain_for_proxy_users', True)
        job_execution = mock.Mock(input_id=1,
                                  output_id=2,
                                  job_id=3,
                                  job_configs={})
        data_source.return_value = mock.Mock(url='swift://container/object')
        self.assertTrue(p.job_execution_requires_proxy_user(job_execution))

        data_source.return_value = mock.Mock(url='')
        job.return_value = mock.Mock(
            mains=[mock.Mock(url='swift://container/object')])
        self.assertTrue(p.job_execution_requires_proxy_user(job_execution))

        job.return_value = mock.Mock(
            mains=[],
            libs=[mock.Mock(url='swift://container/object')])
        self.assertTrue(p.job_execution_requires_proxy_user(job_execution))

        job_execution.job_configs = {'args': ['swift://container/object']}
        job.return_value = mock.Mock(
            mains=[],
            libs=[])
        self.assertTrue(p.job_execution_requires_proxy_user(job_execution))

        job_execution.job_configs = {
            'configs': {'key': 'swift://container/object'}}
        self.assertTrue(p.job_execution_requires_proxy_user(job_execution))

        job_execution.job_configs = {
            'params': {'key': 'swift://container/object'}}
        self.assertTrue(p.job_execution_requires_proxy_user(job_execution))

        data_source_count.return_value = 0
        job_execution.job_configs = {
            'configs': {job_utils.DATA_SOURCE_SUBST_NAME: True}}
        job.return_value = mock.Mock(
            mains=[],
            libs=[])
        self.assertFalse(p.job_execution_requires_proxy_user(job_execution))

        ctx.return_value = 'dummy'
        data_source_count.return_value = 1
        job_execution.job_configs = {
            'configs': {job_utils.DATA_SOURCE_SUBST_NAME: True},
            'args': [job_utils.DATA_SOURCE_PREFIX+'somevalue']}
        self.assertTrue(p.job_execution_requires_proxy_user(job_execution))
        data_source_count.assert_called_with('dummy',
                                             name=('somevalue',),
                                             url='swift://%')
        data_source_count.reset_mock()
        data_source_count.return_value = 1
        myid = uuidutils.generate_uuid()
        job_execution.job_configs = {
            'configs': {job_utils.DATA_SOURCE_SUBST_UUID: True},
            'args': [myid]}
        job.return_value = mock.Mock(
            mains=[],
            libs=[])
        self.assertTrue(p.job_execution_requires_proxy_user(job_execution))
        data_source_count.assert_called_with('dummy',
                                             id=(myid,),
                                             url='swift://%')



# Copyright 2012 New Dream Network, LLC (DreamHost)
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sys
from unittest import mock

import testscenarios
import testtools

from sahara.db.migration import cli


class TestCli(testtools.TestCase):
    func_name = ''
    exp_args = ()
    exp_kwargs = {}

    scenarios = [
        ('stamp',
         dict(argv=['prog', 'stamp', 'foo'], func_name='stamp',
              exp_args=('foo',), exp_kwargs={'sql': False})),
        ('stamp-sql',
         dict(argv=['prog', 'stamp', 'foo', '--sql'], func_name='stamp',
              exp_args=('foo',), exp_kwargs={'sql': True})),
        ('current',
         dict(argv=['prog', 'current'], func_name='current',
              exp_args=[], exp_kwargs=dict())),
        ('history',
         dict(argv=['prog', 'history'], func_name='history',
              exp_args=[], exp_kwargs=dict())),
        ('check_migration',
         dict(argv=['prog', 'check_migration'], func_name='branches',
              exp_args=[], exp_kwargs=dict())),
        ('sync_revision_autogenerate',
         dict(argv=['prog', 'revision', '--autogenerate', '-m', 'message'],
              func_name='revision',
              exp_args=(),
              exp_kwargs={
                  'message': 'message', 'sql': False, 'autogenerate': True})),
        ('sync_revision_sql',
         dict(argv=['prog', 'revision', '--sql', '-m', 'message'],
              func_name='revision',
              exp_args=(),
              exp_kwargs={
                  'message': 'message', 'sql': True, 'autogenerate': False})),
        ('upgrade-sql',
         dict(argv=['prog', 'upgrade', '--sql', 'head'],
              func_name='upgrade',
              exp_args=('head',),
              exp_kwargs={'sql': True})),

        ('upgrade-delta',
         dict(argv=['prog', 'upgrade', '--delta', '3'],
              func_name='upgrade',
              exp_args=('+3',),
              exp_kwargs={'sql': False}))
    ]

    def setUp(self):
        super(TestCli, self).setUp()
        do_alembic_cmd_p = mock.patch.object(cli, 'do_alembic_command')
        self.addCleanup(do_alembic_cmd_p.stop)
        self.do_alembic_cmd = do_alembic_cmd_p.start()
        self.addCleanup(cli.CONF.reset)

    def test_cli(self):
        with mock.patch.object(sys, 'argv', self.argv):
            cli.main()
            self.do_alembic_cmd.assert_has_calls(
                [mock.call(
                    mock.ANY, self.func_name,
                    *self.exp_args, **self.exp_kwargs)]
            )


def load_tests(loader, in_tests, pattern):
    return testscenarios.load_tests_apply_scenarios(loader, in_tests, pattern)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sqlalchemy as sa
import testtools
from unittest import mock

from sahara.db.sqlalchemy import types


class JsonEncodedTest(testtools.TestCase):
    def test_impl(self):
        impl = types.JsonEncoded.impl
        self.assertEqual(sa.Text, impl)

    def test_process_bind_param(self):
        t = types.JsonEncoded()
        self.assertEqual('{"a": 1}', t.process_bind_param({"a": 1}, None))

    def test_process_bind_param_none(self):
        t = types.JsonEncoded()
        self.assertIsNone(t.process_bind_param(None, None))

    def test_process_result_value(self):
        t = types.JsonEncoded()
        self.assertEqual({"a": 1}, t.process_result_value('{"a": 1}', None))

    def test_process_result_value_none(self):
        t = types.JsonEncoded()
        self.assertIsNone(t.process_result_value(None, None))


class MutableDictTest(testtools.TestCase):
    def test_creation(self):
        sample = {"a": 1, "b": 2}
        d = types.MutableDict(sample)
        self.assertEqual(sample, d)

    def test_coerce_dict(self):
        sample = {"a": 1, "b": 2}
        md = types.MutableDict.coerce("test", sample)
        self.assertEqual(sample, md)
        self.assertIsInstance(md, types.MutableDict)

    def test_coerce_mutable_dict(self):
        sample = {"a": 1, "b": 2}
        sample_md = types.MutableDict(sample)
        md = types.MutableDict.coerce("test", sample_md)
        self.assertEqual(sample, md)
        self.assertIs(sample_md, md)

    def test_coerce_unsupported(self):
        with testtools.ExpectedException(ValueError):
            types.MutableDict.coerce("test", list())

    @mock.patch.object(types.MutableDict, 'changed')
    def test_changed_on_update(self, m):
        sample = {"a": 1, "b": 2}
        d = types.MutableDict(sample)
        d.update({"b": 3})
        self.assertEqual({"a": 1, "b": 3}, d)
        self.assertEqual(1, m.call_count)

    @mock.patch.object(types.MutableDict, 'changed')
    def test_changed_on_setitem(self, m):
        sample = {"a": 1, "b": 2}
        d = types.MutableDict(sample)
        d["b"] = 3
        self.assertEqual({"a": 1, "b": 3}, d)
        self.assertEqual(1, m.call_count)

    @mock.patch.object(types.MutableDict, 'changed')
    def test_changed_on_delitem(self, m):
        sample = {"a": 1, "b": 2}
        d = types.MutableDict(sample)
        del d["b"]
        self.assertEqual({"a": 1}, d)
        self.assertEqual(1, m.call_count)


class MutableListTest(testtools.TestCase):
    def test_creation(self):
        sample = [1, 2, 3]
        d = types.MutableList(sample)
        self.assertEqual(sample, d)

    def test_coerce_list(self):
        sample = [1, 2, 3]
        md = types.MutableList.coerce("test", sample)
        self.assertEqual(sample, md)
        self.assertIsInstance(md, types.MutableList)

    def test_coerce_mutable_list(self):
        sample = [1, 2, 3]
        sample_md = types.MutableList(sample)
        md = types.MutableList.coerce("test", sample_md)
        self.assertEqual(sample, md)
        self.assertIs(sample_md, md)

    def test_coerce_unsupported(self):
        with testtools.ExpectedException(ValueError):
            types.MutableList.coerce("test", dict())

    @mock.patch.object(types.MutableList, 'changed')
    def test_changed_on_append(self, m):
        sample = [1, 2, 3]
        lst = types.MutableList(sample)
        lst.append(4)
        self.assertEqual([1, 2, 3, 4], lst)
        self.assertEqual(1, m.call_count)

    @mock.patch.object(types.MutableList, 'changed')
    def test_changed_on_setitem(self, m):
        sample = [1, 2, 3]
        lst = types.MutableList(sample)
        lst[2] = 4
        self.assertEqual([1, 2, 4], lst)
        self.assertEqual(1, m.call_count)

    @mock.patch.object(types.MutableList, 'changed')
    def test_changed_on_delitem(self, m):
        sample = [1, 2, 3]
        lst = types.MutableList(sample)
        del lst[2]
        self.assertEqual([1, 2], lst)
        self.assertEqual(1, m.call_count)



# Copyright (c) 2018 OpenStack Contributors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import testtools
from unittest import mock

import sahara.exceptions as ex
from sahara.service.edp.data_sources.s3.implementation import S3Type
from sahara.tests.unit import base
from sahara.utils.types import FrozenDict


class TestSwiftType(base.SaharaTestCase):
    def setUp(self):
        super(TestSwiftType, self).setUp()
        self.s_type = S3Type()

    def test_validate(self):
        data = {
            "name": "test_data_data_source",
            "type": "s3",
            "url": "s3a://mybucket/myobject",
        }
        self.s_type.validate(data)

        data["url"] = "s3://mybucket/myobject"
        self.s_type.validate(data)

        creds = {}
        data["credentials"] = creds
        self.s_type.validate(data)

        creds["accesskey"] = "key"
        creds["secretkey"] = "key2"
        self.s_type.validate(data)

        creds["bucket_in_path"] = True
        creds["ssl"] = True
        creds["endpoint"] = "blah.org"
        self.s_type.validate(data)

        creds["cool_key"] = "wow"
        with testtools.ExpectedException(ex.InvalidDataException):
            self.s_type.validate(data)

        creds.pop("cool_key")

        creds["ssl"] = "yeah"
        with testtools.ExpectedException(ex.InvalidDataException):
            self.s_type.validate(data)

        creds["ssl"] = True
        creds["bucket_in_path"] = "yeah"
        with testtools.ExpectedException(ex.InvalidDataException):
            self.s_type.validate(data)

    def test_validate_url(self):
        url = ""
        with testtools.ExpectedException(ex.InvalidDataException):
            self.s_type._validate_url(url)
        url = "s3a://"
        with testtools.ExpectedException(ex.InvalidDataException):
            self.s_type._validate_url(url)
        url = "s3a:///"
        with testtools.ExpectedException(ex.InvalidDataException):
            self.s_type._validate_url(url)
        url = "s3a://bucket"
        with testtools.ExpectedException(ex.InvalidDataException):
            self.s_type._validate_url(url)
        url = "s3b://bucket/obj"
        with testtools.ExpectedException(ex.InvalidDataException):
            self.s_type._validate_url(url)
        url = "s3a://bucket/obj"
        self.s_type._validate_url(url)
        url = "s3a://bucket/fold/obj"
        self.s_type._validate_url(url)
        url = "s3a://bucket/obj/"
        self.s_type._validate_url(url)

    def test_prepare_cluster(self):
        ds = mock.Mock()
        cluster = mock.Mock()
        ds.credentials = {}
        job_configs = {}

        self.s_type.prepare_cluster(ds, cluster, job_configs=job_configs)
        self.assertEqual(job_configs, {})

        job_configs['configs'] = {}
        ds.credentials['accesskey'] = 'key'
        self.s_type.prepare_cluster(ds, cluster, job_configs=job_configs)
        self.assertEqual(job_configs['configs'], {'fs.s3a.access.key': 'key'})

        job_configs['configs'] = {'fs.s3a.access.key': 'key2'}
        self.s_type.prepare_cluster(ds, cluster, job_configs=job_configs)
        self.assertEqual(job_configs['configs'], {'fs.s3a.access.key': 'key2'})

        job_configs = FrozenDict({'configs': {}})
        self.s_type.prepare_cluster(ds, cluster, job_configs=job_configs)
        self.assertNotIn(job_configs['configs'], 'accesskey')

        job_configs = {}
        self.s_type.prepare_cluster(ds, cluster, job_configs=job_configs)
        self.assertEqual(job_configs, {})



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import webob.dec

from sahara.api.middleware import auth_valid
from sahara.tests.unit import base as test_base


class AuthValidatorTest(test_base.SaharaTestCase):

    tid1 = '8f9f0c8c4c634d6280e785b44a10b8ab'
    tid2 = '431c8ab1f14f4607bdfc17e05b3924d1'

    @staticmethod
    @webob.dec.wsgify
    def application(req):
        return "Banana"

    def setUp(self):
        super(AuthValidatorTest, self).setUp()
        self.app = auth_valid.AuthValidator(self.application)

    def test_auth_ok_project_id_in_url(self):
        req = webob.Request.blank("/v1.1/%s/clusters" % self.tid1,
                                  accept="text/plain",
                                  method="GET",
                                  environ={"HTTP_X_TENANT_ID": self.tid1})
        res = req.get_response(self.app)
        self.assertEqual(200, res.status_code)

    def test_auth_ok_no_project_id_in_url(self):
        req = webob.Request.blank("/v1.1/clusters",
                                  accept="text/plain",
                                  method="GET",
                                  environ={"HTTP_X_TENANT_ID": self.tid1})
        res = req.get_response(self.app)
        self.assertEqual(200, res.status_code)

    def test_auth_ok_without_path(self):
        req = webob.Request.blank("/", accept="text/plain", method="GET",
                                  environ={"HTTP_X_TENANT_ID": self.tid1})
        res = req.get_response(self.app)
        self.assertEqual(200, res.status_code)

    def test_auth_without_environ(self):
        req = webob.Request.blank("/v1.1/%s/clusters" % self.tid1,
                                  accept="text/plain",
                                  method="GET")
        res = req.get_response(self.app)
        self.assertEqual(503, res.status_code)

    def test_auth_with_wrong_url(self):
        req = webob.Request.blank("/v1.1", accept="text/plain", method="GET",
                                  environ={"HTTP_X_TENANT_ID": self.tid1})
        res = req.get_response(self.app)
        self.assertEqual(404, res.status_code)

    def test_auth_different_tenant(self):
        req = webob.Request.blank("/v1.1/%s/clusters" % self.tid1,
                                  accept="text/plain",
                                  method="GET",
                                  environ={"HTTP_X_TENANT_ID": self.tid2})
        res = req.get_response(self.app)
        self.assertEqual(401, res.status_code)

    def test_auth_tenant_id_in_url_v2(self):
        # NOTE(jfreud): we expect AuthValidator to let this case pass through
        # although Flask will reject it with a 404 further down the pipeline
        req = webob.Request.blank("/v2/%s/clusters" % self.tid1,
                                  accept="text/plain",
                                  method="GET",
                                  environ={"HTTP_X_TENANT_ID": self.tid1})
        res = req.get_response(self.app)
        self.assertEqual(200, res.status_code)



# Copyright (c) 2018 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from sahara.service.castellan import utils as castellan_utils


def delete_secret(id, ctx=None, **kwargs):
    castellan_utils.delete_secret(id, ctx=ctx)


def get_secret(id, ctx=None, **kwargs):
    return castellan_utils.get_secret(id, ctx=ctx)


def store_secret(secret, ctx=None, **kwargs):
    return castellan_utils.store_secret(secret)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import testtools
from unittest import mock

from sahara import exceptions as ex
from sahara.service import validation as v
from sahara.service.validations import clusters as c_val
from sahara.service.validations import clusters_schema as c_schema
from sahara.tests.unit.service.validation import utils as u
from sahara.tests.unit import testutils as tu


class TestClusterDeleteValidation(u.ValidationTestCase):
    def setUp(self):
        super(TestClusterDeleteValidation, self).setUp()
        self.setup_context(tenant_id='tenant1')

    @mock.patch('sahara.service.api.v10.get_cluster')
    def test_cluster_delete_when_protected(self, get_cluster_p):
        cluster = tu.create_cluster("cluster1", "tenant1", "fake",
                                    "0.1", ['ng1'], is_protected=True)
        get_cluster_p.return_value = cluster

        with testtools.ExpectedException(ex.DeletionFailed):
            try:
                c_val.check_cluster_delete(cluster.id)
            except ex.DeletionFailed as e:
                self.assert_protected_resource_exception(e)
                raise e

    @mock.patch('sahara.service.api.v10.get_cluster')
    def test_public_cluster_delete_from_another_tenant(self, get_cluster_p):
        cluster = tu.create_cluster("cluster1", "tenant2", "fake",
                                    "0.1", ['ng1'], is_public=True)
        get_cluster_p.return_value = cluster

        with testtools.ExpectedException(ex.DeletionFailed):
            try:
                c_val.check_cluster_delete(cluster.id)
            except ex.DeletionFailed as e:
                self.assert_created_in_another_tenant_exception(e)
                raise e


class TestClusterDeleteValidationV2(testtools.TestCase):
    @mock.patch("sahara.utils.api.request_data")
    @mock.patch("sahara.utils.api.bad_request")
    def _validate_body(self, request, br, rd):
        m_func = mock.Mock()
        m_func.__name__ = "m_func"

        rd.return_value = request

        validator = v.validate(c_schema.CLUSTER_DELETE_SCHEMA_V2, m_func)
        validator(m_func)(data=request)

        return not br.call_count

    def test_delete_schema_empty_body(self):
        request = {}
        self.assertTrue(self._validate_body(request))

    def test_delete_schema_wrong_type(self):
        request = {"force": "True"}
        self.assertFalse(self._validate_body(request))

    def test_delete_schema_extra_fields(self):
        request = {"force": True, "just_kidding": False}
        self.assertFalse(self._validate_body(request))

    def test_delete_schema_good(self):
        request = {"force": True}
        self.assertTrue(self._validate_body(request))



# Copyright (c) 2017 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import testtools

import sahara.exceptions as ex
from sahara.service.edp.job_binaries import manager as jb_manager
from sahara.tests.unit import base


class JobBinaryManagerSupportTest(base.SaharaTestCase):

    def setUp(self):
        super(JobBinaryManagerSupportTest, self).setUp()
        jb_manager.setup_job_binaries()

    def test_job_binaries_loaded(self):
        jb_types = [jb.name for jb in
                    jb_manager.JOB_BINARIES.get_job_binaries()]

        self.assertIn('internal-db', jb_types)
        self.assertIn('manila', jb_types)
        self.assertIn('swift', jb_types)

    def test_get_job_binary_by_url(self):

        with testtools.ExpectedException(ex.InvalidDataException):
            jb_manager.JOB_BINARIES.get_job_binary_by_url('')

        with testtools.ExpectedException(ex.InvalidDataException):
            jb_manager.JOB_BINARIES.get_job_binary_by_url('internal-db')

        self.assertEqual('internal-db', jb_manager.JOB_BINARIES
                         .get_job_binary_by_url('internal-db://').name)

        self.assertEqual('manila', jb_manager.JOB_BINARIES
                         .get_job_binary_by_url('manila://').name)

        self.assertEqual('swift', jb_manager.JOB_BINARIES
                         .get_job_binary_by_url('swift://').name)

    def test_get_job_binary(self):

        with testtools.ExpectedException(ex.InvalidDataException):
            jb_manager.JOB_BINARIES.get_job_binary('')

        with testtools.ExpectedException(ex.InvalidDataException):
            jb_manager.JOB_BINARIES.get_job_binary('internaldb')

        self.assertEqual('internal-db', jb_manager.JOB_BINARIES
                         .get_job_binary('internal-db').name)

        self.assertEqual('manila', jb_manager.JOB_BINARIES
                         .get_job_binary('manila').name)

        self.assertEqual('swift', jb_manager.JOB_BINARIES
                         .get_job_binary('swift').name)



# Copyright (c) 2016 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg

CONF = cfg.CONF


def has_floating_ip(instance):

    # Alternatively in each of these cases
    # we could use the nova client to look up the
    # ips for the instance and check the attributes
    # to ensure that the management_ip is a floating
    # ip, but a simple comparison with the internal_ip
    # corresponds with the logic in
    # sahara.service.networks.init_instances_ips
    if not instance.node_group.floating_ip_pool:
        return False

    # in the neutron case comparing ips is an extra simple check ...
    # maybe allocation of a floating ip failed for some reason

    # (Alternatively in each of these cases
    # we could use the nova client to look up the
    # ips for the instance and check the attributes
    # to ensure that the management_ip is a floating
    # ip, but a simple comparison with the internal_ip
    # corresponds with the logic in
    # sahara.service.networks.init_instances_ips)
    return instance.management_ip != instance.internal_ip



# Copyright 2014 OpenStack Foundation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""convert clusters.status_description to LongText

Revision ID: 007
Revises: 006
Create Date: 2014-06-20 22:36:00.783444

"""

# revision identifiers, used by Alembic.
revision = '007'
down_revision = '006'

from alembic import op

from sahara.db.sqlalchemy import types as st


def upgrade():
    op.alter_column('clusters', 'status_description',
                    type_=st.LongText(), existing_nullable=True,
                    existing_server_default=None)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

import testtools

from sahara.conductor import resource as r
from sahara import exceptions as ex
from sahara.swift import swift_helper
from sahara.utils import edp


SAMPLE_DICT = {
    'first': [1, 2],
    'second': {'a': 1, 'b': 2}
}

SAMPLE_NESTED_LISTS_DICT = {
    'a': [[{'b': 123}]]
}

SAMPLE_CLUSTER_DICT = {
    'name': 'test-cluster',
    'cluster_configs': {
        'general': {
            'some_overridden_config': 'somevalue'
        }
    },
    'node_groups': [
        {
            'name': 'master',
            'id': 'some_id'
        },
        {
            'id': 'some_id',
            'name': 'worker',
            'node_processes': ['tasktracker', 'datanode'],
            'node_configs': {},
            'instances': [
                {
                    'name': 'test-cluster-001',
                    'ip': '1.1.1.1'
                }
            ]
        }
    ]
}

SAMPLE_JOB_BINARY_DICT = {
    "created_at": "2014-02-14 16:26:08.895897",
    "description": "a job",
    "extra": {
        "password": "password",
        "user": "user"
    },
    "id": "c0caf119-f380-4fab-a46e-0f28ebd23b5c",
    "name": "bob",
    "tenant_id": "6b859fb8d1f44e8eafdfb91f21309b5f",
    "updated_at": "null",
    "url": "swift://bob.sahara/job"
}

SAMPLE_JOB_BINARY_DICT2 = copy.copy(SAMPLE_JOB_BINARY_DICT)
SAMPLE_JOB_BINARY_DICT2["name"] = "bill"
SAMPLE_JOB_BINARY_DICT2["id"] = "c0caf119-1111-2222-a46e-0f28ebd23b5c"
SAMPLE_JOB_BINARY_DICT2["url"] = "swift://bill.sahara/job"

SAMPLE_JOB_DICT = {
    "tenant_id": "test_tenant",
    "name": "job_test",
    "description": "test_desc",
    "type": "Pig",
    "mains": [SAMPLE_JOB_BINARY_DICT],
    "libs": [SAMPLE_JOB_BINARY_DICT2]
}

SAMPLE_DATA_SOURCE = {
    'name': 'input',
    'description': 'some input',
    'type': 'swift',
    'url': 'swift://tmckay.sahara',
    'credentials': {
        'username': 'me',
        'password': 'password'
    }
}

SAMPLE_JOB_EXECUTION = {
    "cluster_id": "7ed1c016-a8a3-4209-9931-6e80f58eea80",
    "created_at": "2014-02-14 17:46:56.631209",
    "extra": {},
    "id": "1b0b1874-a261-4d1f-971a-a2cebadeba6c",
    "info": {
        "actions": [{"conf": "some stuff"},
                    {"conf": "more stuff"}],
        "status": edp.JOB_STATUS_PENDING
    },
    "input_id": "b5ddde55-594e-428f-9040-028be81eb3c2",
    "job_configs": {
        "args": [
            "bob",
            "bill"
        ],
        "configs": {
            swift_helper.HADOOP_SWIFT_PASSWORD: "openstack",
            swift_helper.HADOOP_SWIFT_USERNAME: "admin",
            "myfavoriteconfig": 1
        },
        "proxy_configs": {
            "proxy_username": "admin",
            "proxy_password": "openstack"
        },
        "trusts": {
            "input_id": "9c528755099149b8b7166f3d0fa3bf10",
            "output_id": "3f2bde9d43ec440381dc9f736481e2b0"
        }
    },
    "job_id": "d0f3e397-7bef-42f9-a4db-e5a96059246e",
    "output_id": "f4993830-aa97-4b0b-914a-ab6430f742b6",
    "tenant_id": "6b859fb8d1f44e8eafdfb91f21309b5f"
}


class TestResource(testtools.TestCase):
    def test_resource_creation(self):
        res = r.Resource(SAMPLE_DICT)

        self.assertIsInstance(res.first, list)
        self.assertEqual([1, 2], res.first)
        self.assertIsInstance(res.second, r.Resource)
        self.assertEqual(1, res.second.a)
        self.assertEqual(2, res.second.b)

    def test_resource_immutability(self):
        res = r.Resource(SAMPLE_DICT)

        with testtools.ExpectedException(ex.FrozenClassError):
            res.first.append(123)

        with testtools.ExpectedException(ex.FrozenClassError):
            res.first = 123

        with testtools.ExpectedException(ex.FrozenClassError):
            res.second.a = 123

    def test_nested_lists(self):
        res = r.Resource(SAMPLE_NESTED_LISTS_DICT)
        self.assertEqual(123, res.a[0][0].b)

    def test_cluster_resource(self):
        cluster = r.ClusterResource(SAMPLE_CLUSTER_DICT)

        self.assertEqual('test-cluster', cluster.name)

        self.assertEqual('master', cluster.node_groups[0].name)
        self.assertIsInstance(cluster.node_groups[0], r.NodeGroupResource)
        self.assertEqual('test-cluster', cluster.node_groups[0].cluster.name)

        self.assertEqual('test-cluster-001',
                         cluster.node_groups[1].instances[0].name)
        self.assertIsInstance(cluster.node_groups[1].instances[0],
                              r.InstanceResource)
        self.assertEqual('worker',
                         cluster.node_groups[1].instances[0].node_group.name)

    def test_to_dict(self):
        cluster = r.ClusterResource(SAMPLE_CLUSTER_DICT)
        self.assertEqual(SAMPLE_CLUSTER_DICT, cluster.to_dict())

    def test_to_dict_filtering(self):
        cluster_dict = copy.deepcopy(SAMPLE_CLUSTER_DICT)
        cluster_dict['management_private_key'] = 'abacaba'
        cluster_dict['node_groups'][0]['id'] = 'some_id'

        cluster = r.ClusterResource(cluster_dict)
        self.assertEqual(SAMPLE_CLUSTER_DICT, cluster.to_dict())

    def test_to_wrapped_dict(self):
        cluster = r.ClusterResource(SAMPLE_CLUSTER_DICT)
        wrapped_dict = cluster.to_wrapped_dict()
        self.assertEqual(1, len(wrapped_dict))
        self.assertEqual(SAMPLE_CLUSTER_DICT, wrapped_dict['cluster'])

    def test_job_binary_filter_extra(self):
        job_binary = r.JobBinary(SAMPLE_JOB_BINARY_DICT)
        wrapped_dict = job_binary.to_wrapped_dict()
        self.assertNotIn('extra', wrapped_dict)

    def test_data_source_filter_credentials(self):
        data_source = r.DataSource(SAMPLE_DATA_SOURCE)
        wrapped_dict = data_source.to_wrapped_dict()
        self.assertNotIn('credentials', wrapped_dict)

    def test_job_filter_job_binary(self):
        job = r.Job(SAMPLE_JOB_DICT)
        wrapped_dict = job.to_wrapped_dict()
        self.assertIn('mains', wrapped_dict["job"])
        self.assertIn('libs', wrapped_dict["job"])
        self.assertNotIn('extra', wrapped_dict["job"]['mains'])
        self.assertNotIn('extra', wrapped_dict["job"]['libs'])

    def test_job_execution_filter_credentials(self):
        job_exec = r.JobExecution(SAMPLE_JOB_EXECUTION)
        self.assertIn('extra', job_exec)
        self.assertIn(swift_helper.HADOOP_SWIFT_PASSWORD,
                      job_exec['job_configs']['configs'])
        self.assertIn(swift_helper.HADOOP_SWIFT_USERNAME,
                      job_exec['job_configs']['configs'])
        for a in job_exec['info']['actions']:
            self.assertIn('conf', a)
        self.assertIn('trusts', job_exec['job_configs'])
        self.assertIn('input_id', job_exec['job_configs']['trusts'])
        self.assertIn('output_id', job_exec['job_configs']['trusts'])
        self.assertIn('proxy_configs', job_exec['job_configs'])
        self.assertIn('proxy_username',
                      job_exec['job_configs']['proxy_configs'])
        self.assertIn('proxy_password',
                      job_exec['job_configs']['proxy_configs'])

        wrapped_dict = job_exec.to_wrapped_dict()['job_execution']
        self.assertNotIn('extra', wrapped_dict)

        configs = wrapped_dict['job_configs']['configs']
        self.assertEqual("",
                         configs[swift_helper.HADOOP_SWIFT_PASSWORD])
        self.assertEqual("",
                         configs[swift_helper.HADOOP_SWIFT_USERNAME])

        for a in wrapped_dict['info']['actions']:
            self.assertNotIn('conf', a)

        self.assertNotIn('trusts', wrapped_dict['job_configs'])
        self.assertNotIn('proxy_configs', wrapped_dict['job_configs'])



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from sahara import context
from sahara.tests.unit import base
from sahara.utils.notification import sender


class NotificationTest(base.SaharaTestCase):
    @mock.patch('sahara.utils.rpc.get_notifier')
    def test_update_cluster(self, mock_notify):
        class FakeNotifier(object):
            def info(self, *args):
                self.call = args

        notifier = FakeNotifier()
        mock_notify.return_value = notifier
        ctx = context.ctx()
        sender.status_notify('someId', 'someName', 'someStatus', "update")
        self.expected_args = (ctx,
                              'sahara.cluster.%s' % 'update',
                              {'cluster_id': 'someId',
                               'cluster_name': 'someName',
                               'cluster_status': 'someStatus',
                               'project_id': ctx.tenant_id,
                               'user_id': ctx.user_id})

        self.assertEqual(self.expected_args,
                         notifier.call)



# Copyright (c) 2016 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

import six
import testtools

from sahara import conductor
from sahara import context
from sahara import exceptions
from sahara.plugins import health_check_base
from sahara.service.health import verification_base
from sahara.tests.unit import base
from sahara.tests.unit.conductor import test_api


class Check(health_check_base.BasicHealthCheck):
    def check_health(self):
        return "No criminality"

    def get_health_check_name(self):
        return "James bond check"

    def is_available(self):
        return True


class RedCheck(Check):
    def check_health(self):
        raise health_check_base.RedHealthError("Ooouch!")


class YellowCheck(Check):
    def check_health(self):
        raise health_check_base.YellowHealthError("No problems, boss!")


class TestVerifications(base.SaharaWithDbTestCase):
    def setUp(self):
        super(TestVerifications, self).setUp()
        self.api = conductor.API

    def _cluster_sample(self):
        ctx = context.ctx()
        cluster = self.api.cluster_create(ctx, test_api.SAMPLE_CLUSTER)
        return cluster

    @testtools.skip("Story 2007450 - http://sqlalche.me/e/bhk3")
    @mock.patch('sahara.plugins.health_check_base.get_health_checks')
    def test_verification_start(self, get_health_checks):
        cluster = self._cluster_sample()
        get_health_checks.return_value = [Check]
        verification_base.handle_verification(cluster, {
            'verification': {'status': 'START'}})
        cluster = self.api.cluster_get(context.ctx(), cluster)
        ver = cluster.verification
        self.assertEqual('GREEN', ver['status'])
        self.assertEqual(1, len(ver['checks']))

        self.assertEqual('No criminality', ver.checks[0]['description'])
        id = ver['id']

        get_health_checks.return_value = [YellowCheck, Check, Check]

        verification_base.handle_verification(cluster, {
            'verification': {'status': 'START'}})
        cluster = self.api.cluster_get(context.ctx(), cluster)
        ver = cluster.verification

        self.assertEqual('YELLOW', ver['status'])
        self.assertEqual(3, len(ver['checks']))
        self.assertNotEqual(ver['id'], id)

        get_health_checks.return_value = [RedCheck, YellowCheck]

        verification_base.handle_verification(cluster, {
            'verification': {'status': 'START'}})
        cluster = self.api.cluster_get(context.ctx(), cluster)
        ver = cluster.verification

        self.assertEqual('RED', ver['status'])
        self.assertEqual(2, len(ver['checks']))
        self.assertNotEqual(ver['id'], id)
        self.assertEqual("James bond check", ver['checks'][0]['name'])

    def _validate_exception(self, exc, expected_message):
        message = six.text_type(exc)
        # removing Error ID
        message = message.split('\n')[0]
        self.assertEqual(expected_message, message)

    @testtools.skip("Story 2007450 - http://sqlalche.me/e/bhk3")
    def test_conductor_crud_verifications(self):
        ctx = context.ctx()
        try:
            self.api.cluster_verification_add(
                ctx, '1', values={'status': 'name'})
        except exceptions.NotFoundException as e:
            self._validate_exception(e, "Cluster id '1' not found!")

        cl = self._cluster_sample()
        ver = self.api.cluster_verification_add(
            ctx, cl.id, values={'status': 'GREAT!'})
        ver = self.api.cluster_verification_get(ctx, ver['id'])
        self.assertEqual('GREAT!', ver['status'])

        self.api.cluster_verification_update(ctx, ver['id'],
                                             values={'status': "HEY!"})
        ver = self.api.cluster_verification_get(ctx, ver['id'])
        self.assertEqual('HEY!', ver['status'])
        self.assertIsNone(
            self.api.cluster_verification_delete(ctx, ver['id']))

        try:
            self.api.cluster_verification_delete(ctx, ver['id'])
        except exceptions.NotFoundException as e:
            self._validate_exception(
                e, "Verification id '%s' not found!" % ver['id'])
        try:
            self.api.cluster_verification_update(
                ctx, ver['id'], values={'status': "ONE MORE"})
        except exceptions.NotFoundException as e:
            self._validate_exception(
                e, "Verification id '%s' not found!" % ver['id'])

        self.assertIsNone(self.api.cluster_verification_get(ctx, ver['id']))

    @testtools.skip("Story 2007450 - http://sqlalche.me/e/bhk3")
    def test_conductor_crud_health_checks(self):
        ctx = context.ctx()
        try:
            self.api.cluster_health_check_add(
                ctx, '1', values={'status': 'status'})
        except exceptions.NotFoundException as e:
            self._validate_exception(e, "Verification id '1' not found!")

        cl = self._cluster_sample()
        vid = self.api.cluster_verification_add(
            ctx, cl.id, values={'status': 'GREAT!'})['id']

        hc = self.api.cluster_health_check_add(ctx, vid, {'status': "Sah"})
        hc = self.api.cluster_health_check_get(ctx, hc['id'])
        self.assertEqual('Sah', hc['status'])

        hc = self.api.cluster_health_check_update(
            ctx, hc['id'], {'status': "ara"})
        hc = self.api.cluster_health_check_get(ctx, hc['id'])
        self.assertEqual('ara', hc['status'])

        self.api.cluster_verification_delete(ctx, vid)
        try:
            hc = self.api.cluster_health_check_update(
                ctx, hc['id'], {'status': "rulez!"})
        except exceptions.NotFoundException as e:
            self._validate_exception(
                e, "Health check id '%s' not found!" % hc['id'])

        self.assertIsNone(self.api.cluster_health_check_get(ctx, hc['id']))



# Copyright (c) 2017 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from oslo_utils import uuidutils
import testtools

import sahara.exceptions as ex
from sahara.service.edp.job_binaries.manila import implementation
from sahara.tests.unit import base


class _FakeShare(object):
    def __init__(self, id, share_proto='NFS'):
        self.id = id
        self.share_proto = share_proto


class TestManilaType(base.SaharaTestCase):

    def setUp(self):
        super(TestManilaType, self).setUp()
        self.manila_type = implementation.ManilaType()

    def test_validate_job_location_format(self):

        invalid_url_1 = 'manila://abc'
        invalid_url_2 = 'manila://' + uuidutils.generate_uuid()
        valid_url = 'manila://' + uuidutils.generate_uuid() + '/path'

        self.assertFalse(self.manila_type.validate_job_location_format(''))
        self.assertFalse(self.manila_type.
                         validate_job_location_format(invalid_url_1))
        self.assertFalse(self.manila_type.
                         validate_job_location_format(invalid_url_2))

        self.assertTrue(self.manila_type.
                        validate_job_location_format(valid_url))

    @mock.patch('sahara.service.edp.utils.shares.default_mount')
    @mock.patch('sahara.utils.openstack.manila.client')
    def test_copy_binary_to_cluster(self, f_manilaclient, default_mount):
        cluster_shares = [
            {'id': 'the_share_id',
             'path': '/mnt/mymountpoint'}
        ]

        ng_shares = [
            {'id': 'the_share_id',
             'path': '/mnt/othermountpoint'},
            {'id': '123456',
             'path': '/mnt/themountpoint'}
        ]

        job_binary = mock.Mock()
        job_binary.url = 'manila://the_share_id/the_path'

        remote = mock.Mock()
        remote.instance.node_group.cluster.shares = cluster_shares
        remote.instance.node_group.shares = ng_shares

        info = self.manila_type.copy_binary_to_cluster(job_binary,
                                                       remote=remote)

        self.assertCountEqual('/mnt/mymountpoint/the_path', info)

        job_binary.url = 'manila://123456/the_path'
        info = self.manila_type.copy_binary_to_cluster(job_binary,
                                                       remote=remote)
        self.assertCountEqual('/mnt/themountpoint/the_path', info)

        # missing id
        default_mount.return_value = '/mnt/missing_id'

        job_binary.url = 'manila://missing_id/the_path'
        info = self.manila_type.copy_binary_to_cluster(job_binary,
                                                       remote=remote)

        self.assertCountEqual('/mnt/missing_id/the_path', info)

    @mock.patch('sahara.utils.openstack.manila.client')
    @mock.patch('sahara.conductor.API.cluster_update')
    @mock.patch('sahara.service.edp.utils.shares.mount_shares')
    def test_prepare_cluster(self, mount_shares, cluster_update,
                             f_manilaclient):

        cluster_shares = [
            {'id': 'the_share_id',
             'path': '/mnt/mymountpoint'}
        ]

        ng_shares = [
            {'id': 'the_share_id',
             'path': '/mnt/othermountpoint'},
            {'id': '123456',
             'path': '/mnt/themountpoint'}
        ]

        job_binary = mock.Mock()

        remote = mock.Mock()
        remote.instance.node_group.cluster.shares = cluster_shares
        remote.instance.node_group.shares = ng_shares

        # This should return a default path, and should cause
        # a mount at the default location
        share = _FakeShare("missing_id")
        f_manilaclient.return_value = mock.Mock(shares=mock.Mock(
            get=mock.Mock(return_value=share)))

        job_binary.url = 'manila://missing_id/the_path'

        self.manila_type.prepare_cluster(job_binary, remote=remote)
        self.assertEqual(1, mount_shares.call_count)
        self.assertEqual(1, cluster_update.call_count)

    def test_get_raw_data(self):
        with testtools.ExpectedException(ex.NotImplementedException):
            self.manila_type.get_raw_data({})

    def test_data_validation(self):
        data = {
            "name": "test",
            "url": "man://%s" % uuidutils.generate_uuid(),
            "type": "manila",
            "description": ("incorrect url schema for")
        }
        with testtools.ExpectedException(ex.InvalidDataException):
            self.manila_type.validate(data)

        data = {
            "name": "test",
            "url": "",
            "type": "manila",
            "description": ("empty url")
        }
        with testtools.ExpectedException(ex.InvalidDataException):
            self.manila_type.validate(data)

        data = {
            "name": "test",
            "url": "manila://bob",
            "type": "manila",
            "description": ("netloc is not a uuid")
        }
        with testtools.ExpectedException(ex.InvalidDataException):
            self.manila_type.validate(data)

        data = {
            "name": "test",
            "url": "manila://%s" % uuidutils.generate_uuid(),
            "type": "manila",
            "description": ("netloc is not a uuid")
        }
        with testtools.ExpectedException(ex.InvalidDataException):
            self.manila_type.validate(data)

        data = {
            "name": "test",
            "url": "manila://%s/foo" % uuidutils.generate_uuid(),
            "type": "manila",
            "description": ("correct url")
        }
        self.manila_type.validate(data)



# Copyright (c) 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara import conductor as c
from sahara import context


conductor = c.API


# ClusterTemplate ops

def get_cluster_templates(**kwargs):
    return conductor.cluster_template_get_all(context.ctx(),
                                              regex_search=True, **kwargs)


def get_cluster_template(id):
    return conductor.cluster_template_get(context.ctx(), id)


def create_cluster_template(values):
    return conductor.cluster_template_create(context.ctx(), values)


def terminate_cluster_template(id):
    return conductor.cluster_template_destroy(context.ctx(), id)


def update_cluster_template(id, values):
    return conductor.cluster_template_update(context.ctx(), id, values)


def export_cluster_template(id):
    return conductor.cluster_template_get(context.ctx(), id)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import eventlet


EVENTLET_MONKEY_PATCH_MODULES = dict(os=True,
                                     select=True,
                                     socket=True,
                                     thread=True,
                                     time=True)


def patch_all():
    """Apply all patches.

    List of patches:

    * eventlet's monkey patch for all cases;
    """
    eventlet_monkey_patch()


def eventlet_monkey_patch():
    """Apply eventlet's monkey patch.

    This call should be the first call in application. It's safe to call
    monkey_patch multiple times.
    """
    eventlet.monkey_patch(**EVENTLET_MONKEY_PATCH_MODULES)
    # Monkey patch the original current_thread to use the up-to-date _active
    # global variable. See https://bugs.launchpad.net/bugs/1863021 and
    # https://github.com/eventlet/eventlet/issues/592
    import __original_module_threading as orig_threading
    import threading  # noqa
    orig_threading.current_thread.__globals__['_active'] = threading._active


def eventlet_import_monkey_patched(module):
    """Returns module monkey patched by eventlet.

    It's needed for some tests, for example, context test.
    """
    return eventlet.import_patched(module, **EVENTLET_MONKEY_PATCH_MODULES)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import functools

from oslo_utils import reflection

from sahara import exceptions as ex
from sahara.i18n import _
from sahara.utils import api as u
from sahara.utils import api_validator
from sahara.utils import types


def _get_path(path):
    if path:
        path_string = path[0]
        for x in path[1:]:
            path_string += '[%s]' % str(x)
        return path_string + ': '
    return ''


def _generate_error(errors):
    message = [_get_path(list(e.path)) + e.message for e in errors]
    if message:
        return ex.SaharaException('\n'.join(message), "VALIDATION_ERROR")


def validate_pagination_limit():
    request_args = u.get_request_args()
    if 'limit' in request_args:
        if types.is_int(request_args['limit']):
            if not int(request_args['limit']) > 0:
                raise ex.SaharaException(
                    _("'limit' must be positive integer"), 400)
        else:
            raise ex.SaharaException(
                _("'limit' must be positive integer"), 400)


def get_sorting_field():
    request_args = u.get_request_args()
    if 'sort_by' in request_args:
        sort_by = request_args['sort_by']
        if sort_by:
            sort_by = sort_by[1:] if sort_by[0] == '-' else sort_by
            return sort_by
    return None


def validate_sorting_clusters():
    field = get_sorting_field()
    if field is None:
        return
    if field not in ['id', 'name', 'plugin_name', 'hadoop_version',
                     'status']:
        raise ex.SaharaException(
            _("Unknown field for sorting %s") % field, 400)


def validate_sorting_cluster_templates():
    field = get_sorting_field()
    if field is None:
        return
    if field not in ['id', 'name', 'plugin_name', 'hadoop_version',
                     'created_at', 'updated_at']:
        raise ex.SaharaException(
            _("Unknown field for sorting %s") % field, 400)


def validate_sorting_node_group_templates():
    field = get_sorting_field()
    if field is None:
        return
    if field not in ['id', 'name', 'plugin_name', 'hadoop_version',
                     'created_at', 'updated_at']:
        raise ex.SaharaException(
            _("Unknown field for sorting %s") % field, 400)


def validate_sorting_job_binaries():
    field = get_sorting_field()
    if field is None:
        return
    if field not in ['id', 'name', 'created_at', 'updated_at']:
        raise ex.SaharaException(
            _("Unknown field for sorting %s") % field, 400)


def validate_sorting_job_binary_internals():
    field = get_sorting_field()
    if field is None:
        return
    if field not in ['id', 'name', 'created_at', 'updated_at']:
        raise ex.SaharaException(
            _("Unknown field for sorting %s") % field, 400)


def validate_sorting_data_sources():
    field = get_sorting_field()
    if field is None:
        return
    if field not in ['id', 'name', 'type', 'created_at', 'updated_at']:
        raise ex.SaharaException(
            _("Unknown field for sorting %s") % field, 400)


def validate_sorting_jobs():
    field = get_sorting_field()
    if field is None:
        return
    if field not in ['id', 'name', 'type', 'created_at', 'updated_at']:
        raise ex.SaharaException(
            _("Unknown field for sorting %s") % field, 400)


def validate_sorting_job_executions():
    field = get_sorting_field()
    if field is None:
        return
    if field not in ['id', 'job_template', 'cluster', 'status']:
        raise ex.SaharaException(
            _("Unknown field for sorting %s") % field, 400)


def validate(schema, *validators):
    def decorator(func):
        @functools.wraps(func)
        def handler(*args, **kwargs):
            request_data = u.request_data()
            try:
                if schema:
                    validator = api_validator.ApiValidator(schema)
                    errors = validator.iter_errors(request_data)
                    error = _generate_error(errors)
                    if error:
                        return u.bad_request(error)
                if validators:
                    for validator in validators:
                        validator(**kwargs)
            except ex.SaharaException as e:
                return u.bad_request(e)
            except Exception as e:
                return u.internal_error(
                    500, "Error occurred during validation", e)

            return func(*args, **kwargs)

        return handler

    return decorator


def check_exists(get_func, *id_prop, **get_args):
    def decorator(func):
        @functools.wraps(func)
        def handler(*args, **kwargs):
            if id_prop and not get_args:
                get_args['id'] = id_prop[0]

            if 'marker' in id_prop:
                if 'marker' not in u.get_request_args():
                    return func(*args, **kwargs)
                kwargs['marker'] = u.get_request_args()['marker']

            get_kwargs = {}
            for get_arg in get_args:
                get_kwargs[get_arg] = kwargs[get_args[get_arg]]

            obj = None
            try:
                obj = get_func(**get_kwargs)
            except Exception as e:
                cls_name = reflection.get_class_name(e, fully_qualified=False)
                if 'notfound' not in cls_name.lower():
                    raise e
            if obj is None:
                e = ex.NotFoundException(get_kwargs,
                                         _('Object with %s not found'))
                return u.not_found(e)
            if 'marker' in kwargs:
                del(kwargs['marker'])
            return func(*args, **kwargs)

        return handler

    return decorator


def validate_request_params(supported_params):
    def decorator(func):
        @functools.wraps(func)
        def handler(*args, **kwargs):
            pagination_params = ['marker', 'limit', 'sort_by']
            func_name = func.__name__
            params = u.get_request_args()
            for param in params.keys():
                if (param not in supported_params and
                        param not in pagination_params):
                    return u.invalid_param_error(
                        400,
                        'The only valid params for %s are %s and %s' % (
                            func_name, supported_params, pagination_params))
            return func(*args, **kwargs)
        return handler
    return decorator



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

from sahara.service.api import v10 as api
from sahara.service.validations import node_group_template_schema as nt
from sahara.tests.unit.service.validation import utils as u


def empty(data, **kwargs):
    pass


SAMPLE_DATA = {
    'name': 'a',
    'flavor_id': '42',
    'plugin_name': 'fake',
    'hadoop_version': '0.1',
    'node_processes': ['namenode',
                       'datanode',
                       'secondarynamenode',
                       'nodemanager',
                       'resourcemanager'],
    'node_configs': {
        'HDFS': {
            'mapreduce.task.tmp.dir': '/temp/'
        }
    },
    'image_id': '550e8400-e29b-41d4-a716-446655440000',
    'volumes_per_node': 2,
    'volumes_size': 10,
    'description': 'test node template',
    'floating_ip_pool': 'd9a3bebc-f788-4b81-9a93-aa048022c1ca',
    'is_public': False,
    'is_protected': False
}


class TestNGTemplateUpdateValidation(u.ValidationTestCase):
    def setUp(self):
        super(TestNGTemplateUpdateValidation, self).setUp()
        self._create_object_fun = empty
        self.scheme = nt.NODE_GROUP_TEMPLATE_UPDATE_SCHEMA
        api.plugin_base.setup_plugins()

    def test_node_groups_update_nothing_required(self):
        self._assert_create_object_validation(
            data={}
        )

    def test_ng_template_update_schema(self):
        create = copy.copy(nt.NODE_GROUP_TEMPLATE_SCHEMA)
        update = copy.copy(nt.NODE_GROUP_TEMPLATE_UPDATE_SCHEMA)

        # No required items for update
        self.assertEqual([], update["required"])

        # Other than required, schemas are equal
        del update["required"]
        del create["required"]
        self.assertEqual(create, update)

    def test_ng_template_update_v(self):

        self._assert_create_object_validation(
            data=SAMPLE_DATA
        )

        extra = copy.copy(SAMPLE_DATA)
        extra['dog'] = 'fido'

        self._assert_create_object_validation(
            data=extra,
            bad_req_i=(1, "VALIDATION_ERROR",
                       "Additional properties are not allowed "
                       "('dog' was unexpected)")
        )



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara import context
from sahara.service.edp.binary_retrievers import internal_swift as i_swift
from sahara.service.edp.binary_retrievers import manila_share as manila
from sahara.service.edp.binary_retrievers import s3_storage as s3
from sahara.service.edp.binary_retrievers import sahara_db as db
from sahara.service.edp import s3_common
from sahara.swift import utils as su
from sahara.utils.openstack import manila as m


def get_raw_binary(job_binary, proxy_configs=None,
                   with_context=False, remote=None):
    '''Get the raw data for a job binary

    This will retrieve the raw data for a job binary from it's source. In the
    case of Swift based binaries there is a precedence of credentials for
    authenticating the client. Requesting a context based authentication takes
    precedence over proxy user which takes precedence over embedded
    credentials.

    :param job_binary: The job binary to retrieve
    :param proxy_configs: Proxy user configuration to use as credentials
    :param with_context: Use the current context as credentials
    :param remote: The remote contains node group and cluster information
    :returns: The raw data from a job binary

    '''
    url = job_binary.url
    if url.startswith("internal-db://"):
        res = db.get_raw_data(context.ctx(), job_binary)

    if url.startswith(s3_common.S3_JB_PREFIX):
        res = s3.get_raw_data(job_binary)

    if url.startswith(su.SWIFT_INTERNAL_PREFIX):
        if with_context:
            res = i_swift.get_raw_data_with_context(job_binary)
        else:
            res = i_swift.get_raw_data(job_binary, proxy_configs)

    if url.startswith(m.MANILA_PREFIX):
        res = manila.get_file_info(job_binary, remote)

    return res



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg
from oslo_log import log as logging
from oslo_utils import uuidutils

from sahara import conductor as cond
from sahara import context
from sahara import exceptions as exc
from sahara.i18n import _
from sahara.plugins import provisioning as base
from sahara.plugins import utils as pl_utils
from sahara.service.castellan import utils as key_manager
from sahara.utils import cluster as cl_utils
from sahara.utils import cluster_progress_ops as cpo
from sahara.utils import files


conductor = cond.API
LOG = logging.getLogger(__name__)
CONF = cfg.CONF
POLICY_FILES_DIR = '/tmp/UnlimitedPolicy'


class KDCInstallationFailed(exc.SaharaException):
    code = 'KDC_INSTALL_FAILED'
    message_template = _('KDC installation failed by reason: {reason}')

    def __init__(self, reason):
        message = self.message_template.format(reason=reason)
        super(KDCInstallationFailed, self).__init__(message)


def _config(**kwargs):
    return base.Config(
        applicable_target='Kerberos', priority=1,
        is_optional=True, scope='cluster', **kwargs)


enable_kerberos = _config(
    name='Enable Kerberos Security', config_type='bool',
    default_value=False)
use_existing_kdc = _config(
    name='Existing KDC', config_type='bool',
    default_value=False)
kdc_server_ip = _config(
    name='Server IP of KDC', config_type='string',
    default_value='192.168.0.1',
    description=_('Server IP of KDC server when using existing KDC'))
realm_name = _config(
    name='Realm Name', config_type='string',
    default_value='SAHARA-KDC',
    description=_('The name of realm to be used'))
admin_principal = _config(
    name='Admin principal', config_type='string',
    default_value='sahara/admin',
    description=_('Admin principal for existing KDC server'))
admin_password = _config(
    name='Admin password', config_type='string', default_value='')
policy_url = _config(
    name="JCE libraries", config_type='string',
    default_value=('https://tarballs.openstack.org/sahara-extra/dist/'
                   'common-artifacts/'),
    description=_('Java Cryptography Extension (JCE) '
                  'Unlimited Strength Jurisdiction Policy Files location')
)


def get_config_list():
    return [
        enable_kerberos,
        use_existing_kdc,
        kdc_server_ip,
        realm_name,
        admin_principal,
        admin_password,
        policy_url,
    ]


def get_kdc_host(cluster, server):
    if using_existing_kdc(cluster):
        return "server.%s" % CONF.node_domain
    return server.fqdn()


def is_kerberos_security_enabled(cluster):
    return pl_utils.get_config_value_or_default(
        cluster=cluster, config=enable_kerberos)


def using_existing_kdc(cluster):
    return pl_utils.get_config_value_or_default(
        cluster=cluster, config=use_existing_kdc)


def get_kdc_server_ip(cluster):
    return pl_utils.get_config_value_or_default(
        cluster=cluster, config=kdc_server_ip)


def get_realm_name(cluster):
    return pl_utils.get_config_value_or_default(
        cluster=cluster, config=realm_name)


def get_admin_principal(cluster):
    return pl_utils.get_config_value_or_default(
        cluster=cluster, config=admin_principal)


def get_admin_password(cluster):
    # TODO(vgridnev): support in follow-up improved secret storage for
    # configs
    return pl_utils.get_config_value_or_default(
        cluster=cluster, config=admin_password)


def get_policy_url(cluster):
    return pl_utils.get_config_value_or_default(
        cluster=cluster, config=policy_url)


def setup_clients(cluster, server=None, instances=None):
    if not instances:
        instances = cl_utils.get_instances(cluster)
    server_ip = None
    cpo.add_provisioning_step(
        cluster.id, _("Setting Up Kerberos clients"), len(instances))

    if not server:
        server_ip = get_kdc_server_ip(cluster)
    with context.ThreadGroup() as tg:
        for instance in instances:
            tg.spawn('setup-client-%s' % instance.instance_name,
                     _setup_client_node, cluster, instance,
                     server, server_ip)


def prepare_policy_files(cluster, instances=None):
    if instances is None:
        instances = pl_utils.get_instances(cluster)

    remote_url = get_policy_url(cluster)
    cpo.add_provisioning_step(
        cluster.id, _("Preparing policy files"), len(instances))
    with context.ThreadGroup() as tg:
        for inst in instances:
            tg.spawn(
                'policy-files',
                _prepare_policy_files, inst, remote_url)


def deploy_infrastructure(cluster, server=None):
    if not is_kerberos_security_enabled(cluster):
        LOG.debug("Kerberos security disabled for cluster")
        return
    if not using_existing_kdc(cluster):
        deploy_kdc_server(cluster, server)

    setup_clients(cluster, server)


def _execute_script(client, script):
    with client.remote() as remote:
        script_path = '/tmp/%s' % uuidutils.generate_uuid()[:8]
        remote.write_file_to(script_path, script)
        remote.execute_command('chmod +x %s' % script_path)
        remote.execute_command('bash %s' % script_path)
        remote.execute_command('rm -rf %s' % script_path)


def _get_kdc_config(cluster, os):
    if os == "ubuntu":
        data = files.get_file_text('plugins/resources/kdc_conf')
    else:
        data = files.get_file_text('plugins/resources/kdc_conf_redhat')
    return data % {
        'realm_name': get_realm_name(cluster)
    }


def _get_krb5_config(cluster, server_fqdn):
    data = files.get_file_text('plugins/resources/krb5_config')
    return data % {
        'realm_name': get_realm_name(cluster),
        'server': server_fqdn,
        'node_domain': CONF.node_domain,
    }


def _get_short_uuid():
    return "%s%s" % (uuidutils.generate_uuid()[:8],
                     uuidutils.generate_uuid()[:8])


def get_server_password(cluster):
    if using_existing_kdc(cluster):
        return get_admin_password(cluster)
    ctx = context.ctx()
    cluster = conductor.cluster_get(ctx, cluster)
    extra = cluster.extra.to_dict() if cluster.extra else {}
    passwd_key = 'admin-passwd-kdc'
    if passwd_key not in extra:
        passwd = _get_short_uuid()
        key_id = key_manager.store_secret(passwd, ctx)
        extra[passwd_key] = key_id
        cluster = conductor.cluster_update(ctx, cluster, {'extra': extra})
    passwd = key_manager.get_secret(extra.get(passwd_key), ctx)
    return passwd


def _get_configs_dir(os):
    if os == "ubuntu":
        return "/etc/krb5kdc"
    return "/var/kerberos/krb5kdc"


def _get_kdc_conf_path(os):
    return "%s/kdc.conf" % _get_configs_dir(os)


def _get_realm_create_command(os):
    if os == 'ubuntu':
        return "krb5_newrealm"
    return "kdb5_util create -s"


def _get_acl_config_path(os):
    return "%s/kadm5.acl" % _get_configs_dir(os)


def _get_acl_config():
    return "*/admin * "


def _get_start_command(os, version):
    if os == "ubuntu":
        return ("sudo service krb5-kdc restart && "
                "sudo service krb5-admin-server restart")

    if version.startswith('6'):
        return ("sudo /etc/rc.d/init.d/krb5kdc start "
                "&& sudo /etc/rc.d/init.d/kadmin start")

    if version.startswith('7'):
        return ("sudo systemctl start krb5kdc &&"
                "sudo systemctl start kadmin")

    raise ValueError(
        _("Unable to get kdc server start command"))


def _get_server_installation_script(cluster, server_fqdn, os, version):
    data = files.get_file_text(
        'plugins/resources/mit-kdc-server-init.sh.template')
    return data % {
        'kdc_conf': _get_kdc_config(cluster, os),
        'kdc_conf_path': _get_kdc_conf_path(os),
        'acl_conf': _get_acl_config(),
        'acl_conf_path': _get_acl_config_path(os),
        'realm_create': _get_realm_create_command(os),
        'krb5_conf': _get_krb5_config(cluster, server_fqdn),
        'admin_principal': get_admin_principal(cluster),
        'password': get_server_password(cluster),
        'os': os,
        'start_command': _get_start_command(os, version),
    }


@cpo.event_wrapper(True, step=_("Deploy KDC server"), param=('cluster', 0))
def deploy_kdc_server(cluster, server):
    with server.remote() as r:
        os = r.get_os_distrib()
        version = r.get_os_version()
    script = _get_server_installation_script(
        cluster, server.fqdn(), os, version)
    _execute_script(server, script)


def _push_etc_hosts_entry(client, entry):
    with client.remote() as r:
        r.execute_command('echo %s | sudo tee -a /etc/hosts' % entry)


def _get_client_installation_script(cluster, server_fqdn, os):
    data = files.get_file_text('plugins/resources/krb-client-init.sh.template')
    return data % {
        'os': os,
        'krb5_conf': _get_krb5_config(cluster, server_fqdn),
    }


@cpo.event_wrapper(True, param=('client', 1))
def _setup_client_node(cluster, client, server=None, server_ip=None):
    if server:
        server_fqdn = server.fqdn()
    elif server_ip:
        server_fqdn = "server." % CONF.node_domain
        _push_etc_hosts_entry(
            client, "%s %s %s" % (server_ip, server_fqdn, server))
    else:
        raise KDCInstallationFailed(_('Server or server ip are not provided'))
    with client.remote() as r:
        os = r.get_os_distrib()
    script = _get_client_installation_script(cluster, server_fqdn, os)
    _execute_script(client, script)


@cpo.event_wrapper(True)
def _prepare_policy_files(instance, remote_url):
    with instance.remote() as r:
        cmd = 'cut -f2 -d \"=\" /etc/profile.d/99-java.sh | head -1'
        exit_code, java_home = r.execute_command(cmd)
        java_home = java_home.strip()
        results = [
            r.execute_command(
                "ls %s/local_policy.jar" % POLICY_FILES_DIR,
                raise_when_error=False)[0] != 0,
            r.execute_command(
                "ls %s/US_export_policy.jar" % POLICY_FILES_DIR,
                raise_when_error=False)[0] != 0
        ]
        # a least one exit code is not zero
        if any(results):
            r.execute_command('mkdir %s' % POLICY_FILES_DIR)
            r.execute_command(
                "sudo curl %s/local_policy.jar -o %s/local_policy.jar" % (
                    remote_url, POLICY_FILES_DIR))
            r.execute_command(
                "sudo curl %s/US_export_policy.jar -o "
                "%s/US_export_policy.jar" % (
                    remote_url, POLICY_FILES_DIR))
        r.execute_command(
            'sudo cp %s/*.jar %s/lib/security/'
            % (POLICY_FILES_DIR, java_home))


def _get_script_for_user_creation(cluster, instance, user):
    data = files.get_file_text(
        'plugins/resources/create-principal-keytab')
    cron_file = files.get_file_text('plugins/resources/cron-file')
    cron_script = files.get_file_text('plugins/resources/cron-script')
    data = data % {
        'user': user, 'admin_principal': get_admin_principal(cluster),
        'admin_password': get_server_password(cluster),
        'principal': "%s/sahara-%s@%s" % (
            user, instance.fqdn(), get_realm_name(cluster)),
        'keytab': '%s-sahara-%s.keytab' % (user, instance.fqdn())
    }
    cron_script_location = '/tmp/sahara-kerberos/%s.sh' % _get_short_uuid()
    cron_file = cron_file % {'refresher': cron_script_location, 'user': user}
    cron_script = cron_script % {
        'principal': "%s/sahara-%s@%s" % (
            user, instance.fqdn(), get_realm_name(cluster)),
        'keytab': '%s-sahara-%s.keytab' % (user, instance.fqdn()),
        'user': user,
    }
    return data, cron_file, cron_script, cron_script_location


def _create_keytabs_for_user(instance, user):
    script, cron, cron_script, cs_location = _get_script_for_user_creation(
        instance.cluster, instance, user)
    _execute_script(instance, script)
    # setting up refresher
    with instance.remote() as r:
        tmp_location = '/tmp/%s' % _get_short_uuid()
        r.write_file_to(tmp_location, cron_script, run_as_root=True)
        r.execute_command(
            "cat {0} | sudo tee {1} "
            "&& rm -rf {0} && sudo chmod +x {1}".format(
                tmp_location, cs_location))
        r.execute_command(
            'echo "%s" | sudo tee /etc/cron.d/%s.cron' % (
                cron, _get_short_uuid()))
        # executing script
        r.execute_command('sudo bash %s' % cs_location)


@cpo.event_wrapper(
    True, step=_('Setting up keytabs for users'), param=('cluster', 0))
def create_keytabs_for_map(cluster, mapper):
    # cluster parameter is used by event log feature
    with context.ThreadGroup() as tg:
        for user, instances in mapper.items():
            for instance in instances:
                tg.spawn(
                    'create-keytabs', _create_keytabs_for_user,
                    instance, user)
